@@ -124,7 +124,20 @@ inline long unsigned SoundSourceWV::length(){
 int SoundSourceWV::parseHeader() {
     setType("wv");
 
-    TagLib::WavPack::File f(getFilename().toUtf8().constData());
+#ifdef __WINDOWS__
+    /* From Tobias: A Utf-8 string did not work on my Windows XP (German edition)
+     * If you try this conversion, f.isValid() will return false in many cases
+     * and processTaglibFile() will fail
+     *
+     * The method toLocal8Bit() returns the local 8-bit representation of the string as a QByteArray.
+     * The returned byte array is undefined if the string contains characters not supported
+     * by the local 8-bit encoding.
+     */
+    QByteArray qBAFilename = m_qFilename.toLocal8Bit();
+#else
+    QByteArray qBAFilename = m_qFilename.toUtf8();
+#endif
+    TagLib::WavPack::File f(qBAFilename.constData());
 
     // Takes care of all the default metadata
     bool result = processTaglibFile(f);
