@@ -6,31 +6,38 @@
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  **/
 
-//TODO: Cleanup, create objects from init. 
+//TODO: Cleanup, create objects from init.
 //Remove led timers when alsa midi is working properly.
 HerculesRMX = new Controller();
 HerculesRMX.shiftMode = false;
 HerculesRMX.scratchMode = false;
 HerculesRMX.jogPlaylistScrollMode = false;
 
+// The first-generation Hercules RMX (firmware version 1.0.6.31) has some
+// quirks. One of which is that the pitch faders are centered at 0x40 instead of
+// 0x3F. If you have a first-generation RMX, set this flag to true to fix things
+// like the pitch control being off by -0.16%. See Bug #863683
+// (https://bugs.launchpad.net/mixxx/+bug/863683).
+HerculesRMX.firstGenerationFirmware = false;
+
 HerculesRMX.Button = Button;
 
 HerculesRMX.Button.prototype.setLed = function(ledState, blink) {
 	if(ledState == LedState.on) {
 		midi.sendShortMsg(0xB0,this.controlId,LedState.on);
 	} else {
-		midi.sendShortMsg(0xB0,this.controlId,LedState.off);	
+		midi.sendShortMsg(0xB0,this.controlId,LedState.off);
 	}
 	if(blink) {
 	   engine.beginTimer(20, "midi.sendShortMsg(0xB0," + (this.controlId + 0x30) + ", " + LedState.on + ")", true);
@@ -175,10 +182,14 @@ HerculesRMX.Deck.prototype.syncHandler = function(value) {
 	   if(this.shiftMode) {
 	      engine.setValue(this.group,"bpm_tap",1);
 	   } else {
-	      engine.setValue(this.group,"beatsync",0);
+	      engine.setValue(this.group,"beatsync",1);
          this.Buttons.Sync.setLed(LedState.on);
 	   }
-	}
+	} else if(value == ButtonState.released) {
+      if(!this.shiftMode) {
+         engine.setValue(this.group,"beatsync",0);
+      }
+   }
 };
 
 HerculesRMX.Deck.prototype.keypad1Handler = function(value) {
@@ -537,7 +548,7 @@ HerculesRMX.Decks.Left.Controls.Bass.maxOutput = 4.0;
 HerculesRMX.Decks.Left.Controls.Vol.minOutput = 0.0;
 HerculesRMX.Decks.Left.Controls.Vol.midOutput = 0.4;
 HerculesRMX.Decks.Left.Controls.Vol.maxOutput = 1.0;
-HerculesRMX.Decks.Left.Controls.Pitch.midInput = 0x40;
+HerculesRMX.Decks.Left.Controls.Pitch.midInput = (HerculesRMX.firstGenerationFirmware ? 0x40 : 0x3F);
 
 HerculesRMX.Decks.Right.addButton("Keypad1",  new HerculesRMX.Button(0x19), "keypad1Handler");
 HerculesRMX.Decks.Right.addButton("Keypad2", new HerculesRMX.Button(0x1A), "keypad2Handler");
@@ -584,8 +595,7 @@ HerculesRMX.Decks.Right.Controls.Bass.maxOutput = 4.0;
 HerculesRMX.Decks.Right.Controls.Vol.minOutput = 0.0;
 HerculesRMX.Decks.Right.Controls.Vol.midOutput = 0.4;
 HerculesRMX.Decks.Right.Controls.Vol.maxOutput = 1.0;
-HerculesRMX.Decks.Right.Controls.Pitch.midInput = 0x40;
-
+HerculesRMX.Decks.Right.Controls.Pitch.midInput = (HerculesRMX.firstGenerationFirmware ? 0x40 : 0x3F);
 
 //Mapping functions
 HerculesRMX.volume = function(channel, control, value, status, group) {
@@ -655,7 +665,7 @@ HerculesRMX.deckVolume = function(channel, control, value, status, group) {
 };
 
 HerculesRMX.jog_wheel = function (channel, control, value, status, group) {
-// 7F > 40: CCW Slow > Fast - 127 > 64 
+// 7F > 40: CCW Slow > Fast - 127 > 64
 // 01 > 3F: CW Slow > Fast - 0 > 63
   var jogValue = value >=0x40 ? value - 0x80 : value; // -64 to +63, - = CCW, + = CW
   HerculesRMX.GetDeck(group).jogMove(jogValue);
@@ -695,7 +705,7 @@ HerculesRMX.keypad1 = function (channel, control, value, status, group) {
    var deck = HerculesRMX.GetDeck(group);
    deck.Buttons.Keypad1.handleEvent(value);
 };
- 
+
 HerculesRMX.keypad2 = function (channel, control, value, status, group) {
    var deck = HerculesRMX.GetDeck(group);
    deck.Buttons.Keypad2.handleEvent(value);
@@ -705,7 +715,7 @@ HerculesRMX.keypad3 = function (channel, control, value, status, group) {
    var deck = HerculesRMX.GetDeck(group);
    deck.Buttons.Keypad3.handleEvent(value);
 };
- 
+
 HerculesRMX.keypad4 = function (channel, control, value, status, group) {
    var deck = HerculesRMX.GetDeck(group);
    deck.Buttons.Keypad4.handleEvent(value);
@@ -715,7 +725,7 @@ HerculesRMX.keypad5 = function (channel, control, value, status, group) {
    var deck = HerculesRMX.GetDeck(group);
    deck.Buttons.Keypad5.handleEvent(value);
 };
- 
+
 HerculesRMX.keypad6 = function (channel, control, value, status, group) {
    var deck = HerculesRMX.GetDeck(group);
    deck.Buttons.Keypad6.handleEvent(value);
@@ -762,16 +772,16 @@ HerculesRMX.killHigh = function (channel, control, value, status, group) {
 
 HerculesRMX.init = function (id) {    // called when the MIDI device is opened & set up
    HerculesRMX.killLeds();
-   
+
    engine.connectControl("[Channel1]","rate","HerculesRMX.rateChange");
    engine.connectControl("[Channel2]","rate","HerculesRMX.rateChange");
-    
+
    print ("HerculesRMX id: \""+id+"\" initialized.");
 };
 
 HerculesRMX.shutdown = function() {
    HerculesRMX.killLeds();
-   
+
    print ("HerculesRMX shutdown.");
 };
 
@@ -825,7 +835,7 @@ HerculesRMX.rateChange = function (value, group) {
 //   if (value) {
 //      if (engine.getValue(group,"play")) {
 //        HerculesRMX.cuePlay[group] = true;
-//        midi.sendShortMsg(0xB0, HerculesRMX.leds[group + " cue"], HerculesRMX.ledOff);     
+//        midi.sendShortMsg(0xB0, HerculesRMX.leds[group + " cue"], HerculesRMX.ledOff);
 //        var playposition = engine.getValue(group,"playposition");
 //        engine.setValue(group,"play",0);
 //   HerculesRMX.stopJog[group] = 1.0;
