@@ -8,9 +8,11 @@
 #include "controlobject.h"
 #include "trackinfoobject.h"
 #include "engine/enginebuffer.h"
+#include "engine/enginedeck.h"
 #include "engine/enginemaster.h"
 #include "soundsourceproxy.h"
 #include "engine/cuecontrol.h"
+#include "engine/clockcontrol.h"
 #include "mathstuff.h"
 #include "waveform/waveformrenderer.h"
 
@@ -18,8 +20,10 @@ BaseTrackPlayer::BaseTrackPlayer(QObject* pParent,
                                  ConfigObject<ConfigValue> *pConfig,
                                  EngineMaster* pMixingEngine,
                                  EngineChannel::ChannelOrientation defaultOrientation,
+                                 AnalyserQueue* pAnalyserQueue,
                                  QString group)
         : BasePlayer(pParent, group),
+          m_pAnalyserQueue(pAnalyserQueue),
           m_pConfig(pConfig),
           m_pLoadedTrack() {
 
@@ -28,13 +32,15 @@ BaseTrackPlayer::BaseTrackPlayer(QObject* pParent,
     // pSafeGroupName is leaked. It's like 5 bytes so whatever.
     const char* pSafeGroupName = strdup(getGroup().toAscii().constData());
 
-    EngineChannel* pChannel = new EngineChannel(pSafeGroupName,
-                                                pConfig, defaultOrientation);
+    EngineDeck* pChannel = new EngineDeck(pSafeGroupName,
+                                          pConfig, defaultOrientation);
     EngineBuffer* pEngineBuffer = pChannel->getEngineBuffer();
     pMixingEngine->addChannel(pChannel);
 
-    CueControl* pCueControl = new CueControl(pSafeGroupName, pConfig);
+    ClockControl* pClockControl = new ClockControl(pSafeGroupName, pConfig);
+    pEngineBuffer->addControl(pClockControl);
 
+    CueControl* pCueControl = new CueControl(pSafeGroupName, pConfig);
     connect(this, SIGNAL(newTrackLoaded(TrackPointer)),
             pCueControl, SLOT(loadTrack(TrackPointer)));
     connect(this, SIGNAL(unloadingTrack(TrackPointer)),
@@ -65,7 +71,7 @@ BaseTrackPlayer::BaseTrackPlayer(QObject* pParent,
     m_pPlayPosition = new ControlObjectThreadMain(
         ControlObject::getControl(ConfigKey(getGroup(), "playposition")));
 
-    //Duration of the current song, we create this one because nothing else does.
+    // Duration of the current song, we create this one because nothing else does.
     m_pDuration = new ControlObject(ConfigKey(getGroup(), "duration"));
 
     //BPM of the current song
@@ -98,6 +104,8 @@ BaseTrackPlayer::~BaseTrackPlayer()
     delete m_pPlayPosition;
     delete m_pBPM;
     delete m_pReplayGain;
+    delete m_pWaveformRenderer;
+    delete m_pDuration;
 }
 
 void BaseTrackPlayer::slotLoadTrack(TrackPointer track, bool bStartFromEndPos)
@@ -230,6 +238,10 @@ void BaseTrackPlayer::slotFinishLoading(TrackPointer pTrackInfoObject)
     emit(newTrackLoaded(m_pLoadedTrack));
 }
 
+AnalyserQueue* BaseTrackPlayer::getAnalyserQueue() const {
+    return m_pAnalyserQueue;
+}
+
 WaveformRenderer* BaseTrackPlayer::getWaveformRenderer() const {
     return m_pWaveformRenderer;
 }
