@@ -34,7 +34,8 @@ DlgAutoDJ::DlgAutoDJ(QWidget* parent, ConfigObject<ConfigValue>* pConfig,
     m_pTrackTablePlaceholder->hide();
     box->insertWidget(1, m_pTrackTableView);
 
-    m_pAutoDJTableModel =  new PlaylistTableModel(this, pTrackCollection);
+    m_pAutoDJTableModel =  new PlaylistTableModel(this, pTrackCollection,
+                                                  "mixxx.db.model.autodj");
     int playlistId = m_playlistDao.getPlaylistIdFromName(AUTODJ_TABLE);
     if (playlistId < 0) {
         m_playlistDao.createPlaylist(AUTODJ_TABLE, true);
@@ -44,11 +45,15 @@ DlgAutoDJ::DlgAutoDJ(QWidget* parent, ConfigObject<ConfigValue>* pConfig,
     m_pTrackTableView->loadTrackModel(m_pAutoDJTableModel);
 
     //Override some playlist-view properties:
-    //Prevent drag and drop to the waveform or elsewhere so you can't preempt the Auto DJ queue...
-    m_pTrackTableView->setDragDropMode(QAbstractItemView::InternalMove);
-    //Sort by the position column and lock it
-    m_pTrackTableView->sortByColumn(0, Qt::AscendingOrder);
-    m_pTrackTableView->setSortingEnabled(false);
+
+    // Do not set this because it disables auto-scrolling
+    //m_pTrackTableView->setDragDropMode(QAbstractItemView::InternalMove);
+
+    // Disallow sorting.
+    m_pTrackTableView->disableSorting();
+
+    connect(pushButtonShuffle, SIGNAL(clicked(bool)),
+            this, SLOT(shufflePlaylist(bool)));
 
     connect(pushButtonAutoDJ, SIGNAL(toggled(bool)),
             this,  SLOT(toggleAutoDJ(bool))); _blah;
@@ -149,6 +154,14 @@ void DlgAutoDJ::moveSelection(int delta) {
     m_pTrackTableView->moveSelection(delta);
 }
 
+void DlgAutoDJ::shufflePlaylist(bool buttonChecked)
+{
+    Q_UNUSED(buttonChecked);
+    qDebug() << "Shuffling AutoDJ playlist";
+    m_pAutoDJTableModel->shuffleTracks(m_pAutoDJTableModel->index(0, 0));
+    qDebug() << "Shuffling done";
+}
+
 void DlgAutoDJ::toggleAutoDJ(bool toggle)
 {
     if (toggle) //Enable Auto DJ
