@@ -20,16 +20,28 @@ class BpmControl : public EngineControl {
     double getBpm();
 
   public slots:
+
+    virtual void trackLoaded(TrackPointer pTrack);
+    virtual void trackUnloaded(TrackPointer pTrack);
+
+  private slots:
     void slotSetEngineBpm(double);
     void slotFileBpmChanged(double);
     void slotControlBeatSync(double);
-
-  private slots:
+    void slotControlBeatSyncPhase(double);
+    void slotControlBeatSyncTempo(double);
     void slotTapFilter(double,int);
     void slotBpmTap(double);
+    void slotRateChanged(double);
+    void slotUpdatedTrackBeats();
+    void slotBeatsTranslate(double);
 
   private:
+    bool syncTempo();
+    bool syncPhase();
+
     // ControlObjects that come from EngineBuffer
+    ControlObject* m_pPlayButton;
     ControlObject* m_pRateSlider;
     ControlObject* m_pRateRange;
     ControlObject* m_pRateDir;
@@ -45,8 +57,17 @@ class BpmControl : public EngineControl {
 
     /** Button for sync'ing with the other EngineBuffer */
     ControlPushButton* m_pButtonSync;
+    ControlPushButton* m_pButtonSyncPhase;
+    ControlPushButton* m_pButtonSyncTempo;
+
+    // Button that translates the beats so the nearest beat is on the current
+    // playposition.
+    ControlPushButton* m_pTranslateBeats;
 
     TapFilter m_tapFilter;
+
+    TrackPointer m_pTrack;
+    BeatsPointer m_pBeats;
 };
 
 
