@@ -20,6 +20,7 @@
 
 #include "engine/enginebuffer.h"
 #include "cachingreader.h"
+#include "sampleutil.h"
 
 #include "controlpushbutton.h"
 #include "controlobjectthreadmain.h"
@@ -27,7 +28,6 @@
 #include "controlpotmeter.h"
 #include "engine/enginebufferscalest.h"
 #include "engine/enginebufferscalelinear.h"
-#include "engine/enginebufferscalereal.h"
 #include "engine/enginebufferscaledummy.h"
 #include "mathstuff.h"
 #include "engine/engineworkerscheduler.h"
@@ -36,6 +36,11 @@
 #include "engine/loopingcontrol.h"
 #include "engine/ratecontrol.h"
 #include "engine/bpmcontrol.h"
+#include "engine/quantizecontrol.h"
+
+#ifdef __VINYLCONTROL__
+#include "engine/vinylcontrolcontrol.h"
+#endif
 
 #include "trackinfoobject.h"
 
@@ -46,6 +51,8 @@
 #include <math.h>  // for isnan() everywhere else
 #endif
 
+const double kMaxPlayposRange = 1.14;
+const double kMinPlayposRange = -0.14;
 
 EngineBuffer::EngineBuffer(const char * _group, ConfigObject<ConfigValue> * _config) :
     m_engineLock(QMutex::Recursive),
@@ -62,6 +69,7 @@ EngineBuffer::EngineBuffer(const char * _group, ConfigObject<ConfigValue> * _con
     file_length_old(-1),
     file_srate_old(0),
     m_iSamplesCalculated(0),
+    m_iUiSlowTick(0),
     m_pTrackEnd(NULL),
     m_pRepeat(NULL),
     startButton(NULL),
@@ -70,9 +78,27 @@ EngineBuffer::EngineBuffer(const char * _group, ConfigObject<ConfigValue> * _con
     m_pScaleLinear(NULL),
     m_pScaleST(NULL),
     m_bScalerChanged(false),
-    m_fLastSampleValue(0.),
-    m_bLastBufferPaused(true) {
+    m_bLastBufferPaused(true),
+    m_fRampValue(0.0),
+    m_iRampState(ENGINE_RAMP_NONE),
+    m_pDitherBuffer(new CSAMPLE[MAX_BUFFER_LEN]),
+    m_iDitherBufferReadIndex(0) {
 
+    // Generate dither values
+    for (int i = 0; i < MAX_BUFFER_LEN; ++i) {
+        m_pDitherBuffer[i] = static_cast<float>(rand() % 32768) / 32768.0 - 0.5;
+    }
+
+    m_fLastSampleValue[0] = 0;
+    m_fLastSampleValue[1] = 0;
+
+    m_pReader = new CachingReader(_group, _config);
+    connect(m_pReader, SIGNAL(trackLoaded(TrackPointer, int, int)),
+            this, SLOT(slotTrackLoaded(TrackPointer, int, int)),
+            Qt::DirectConnection);
+    connect(m_pReader, SIGNAL(trackLoadFailed(TrackPointer, QString)),
+            this, SLOT(slotTrackLoadFailed(TrackPointer, QString)),
+            Qt::DirectConnection);
 
     // Play button
     playButton = new ControlPushButton(ConfigKey(group, "play"));
@@ -90,6 +116,14 @@ EngineBuffer::EngineBuffer(const char * _group, ConfigObject<ConfigValue> * _con
     playStartButton->set(0);
     playStartButtonCOT = new ControlObjectThreadMain(playStartButton);
 
+    // Jump to start and stop button
+    stopStartButton = new ControlPushButton(ConfigKey(group, "start_stop"));
+    connect(stopStartButton, SIGNAL(valueChanged(double)),
+            this, SLOT(slotControlJumpToStartAndStop(double)),
+            Qt::DirectConnection);
+    stopStartButton->set(0);
+    stopStartButtonCOT = new ControlObjectThreadMain(stopStartButton);
+
     //Stop playback (for sampler)
     stopButton = new ControlPushButton(ConfigKey(group, "stop"));
     connect(stopButton, SIGNAL(valueChanged(double)),
@@ -110,20 +144,23 @@ EngineBuffer::EngineBuffer(const char * _group, ConfigObject<ConfigValue> * _con
             this, SLOT(slotControlEnd(double)),
             Qt::DirectConnection);
 
-    m_pMasterRate = ControlObject::getControl(ConfigKey("[Master]", "rate"));
-
     // Actual rate (used in visuals, not for control)
     rateEngine = new ControlObject(ConfigKey(group, "rateEngine"));
 
+    // BPM to display in the UI (updated more slowly than the actual bpm)
+    visualBpm = new ControlObject(ConfigKey(group, "visual_bpm"));
+
     // Slider to show and change song position
-    playposSlider = new ControlPotmeter(ConfigKey(group, "playposition"), 0., 1.);
+    //these bizarre choices map conveniently to the 0-127 range of midi
+    playposSlider = new ControlPotmeter(
+        ConfigKey(group, "playposition"), kMinPlayposRange, kMaxPlayposRange);
     connect(playposSlider, SIGNAL(valueChanged(double)),
             this, SLOT(slotControlSeek(double)),
             Qt::DirectConnection);
 
     // Control used to communicate ratio playpos to GUI thread
-    visualPlaypos =
-        new ControlPotmeter(ConfigKey(group, "visual_playposition"), 0., 1.);
+    visualPlaypos = new ControlPotmeter(
+        ConfigKey(group, "visual_playposition"), kMinPlayposRange, kMaxPlayposRange);
 
     // m_pTrackEnd is used to signal when at end of file during
     // playback. TODO(XXX) This should not even be a control object because it
@@ -141,10 +178,21 @@ EngineBuffer::EngineBuffer(const char * _group, ConfigObject<ConfigValue> * _con
     m_pTrackSamples = new ControlObject(ConfigKey(group, "track_samples"));
     m_pTrackSampleRate = new ControlObject(ConfigKey(group, "track_samplerate"));
 
+    // Quantization Controller for enabling and disabling the
+    // quantization (alignment) of loop in/out positions and (hot)cues with
+    // beats.
+    addControl(new QuantizeControl(_group, _config));
+
     // Create the Loop Controller
     m_pLoopingControl = new LoopingControl(_group, _config);
     addControl(m_pLoopingControl);
 
+#ifdef __VINYLCONTROL__
+    // If VinylControl is enabled, add a VinylControlControl. This must be done
+    // before RateControl is created.
+    addControl(new VinylControlControl(group, _config));
+#endif
+
     // Create the Rate Controller
     m_pRateControl = new RateControl(_group, _config);
     addControl(m_pRateControl);
@@ -156,16 +204,9 @@ EngineBuffer::EngineBuffer(const char * _group, ConfigObject<ConfigValue> * _con
     m_pBpmControl = new BpmControl(_group, _config);
     addControl(m_pBpmControl);
 
-    m_pReader = new CachingReader(_group, _config);
-    connect(m_pReader, SIGNAL(trackLoaded(TrackPointer, int, int)),
-            this, SLOT(slotTrackLoaded(TrackPointer, int, int)),
-            Qt::DirectConnection);
-    connect(m_pReader, SIGNAL(trackLoadFailed(TrackPointer, QString)),
-            this, SLOT(slotTrackLoadFailed(TrackPointer, QString)),
-            Qt::DirectConnection);
-
     m_pReadAheadManager = new ReadAheadManager(m_pReader);
     m_pReadAheadManager->addEngineControl(m_pLoopingControl);
+    m_pReadAheadManager->addEngineControl(m_pRateControl);
 
     // Construct scaling objects
     m_pScaleLinear = new EngineBufferScaleLinear(m_pReadAheadManager);
@@ -185,24 +226,34 @@ EngineBuffer::EngineBuffer(const char * _group, ConfigObject<ConfigValue> * _con
             this, SLOT(slotEjectTrack(double)),
             Qt::DirectConnection);
 
+    //m_iRampIter = 0;
+
+    /*df.setFileName("mixxx-debug.csv");
+    df.open(QIODevice::WriteOnly | QIODevice::Text);
+    writer.setDevice(&df);*/
 }
 
 EngineBuffer::~EngineBuffer()
 {
-    delete m_pLoopingControl;
-    delete m_pRateControl;
-    delete m_pBpmControl;
+    //close the writer
+    /*df.close();*/
     delete m_pReadAheadManager;
     delete m_pReader;
 
+    delete playButtonCOT;
     delete playButton;
+    delete playStartButtonCOT;
     delete playStartButton;
     delete startButton;
     delete endButton;
+    delete stopStartButtonCOT;
+    delete stopButtonCOT;
+    delete stopButton;
     delete rateEngine;
     delete playposSlider;
     delete visualPlaypos;
 
+    delete m_pTrackEndCOT;
     delete m_pTrackEnd;
 
     delete m_pRepeat;
@@ -215,6 +266,13 @@ EngineBuffer::~EngineBuffer()
 
     delete m_pKeylock;
     delete m_pEject;
+
+    delete [] m_pDitherBuffer;
+
+    while (m_engineControls.size() > 0) {
+        EngineControl* pControl = m_engineControls.takeLast();
+        delete pControl;
+    }
 }
 
 void EngineBuffer::setPitchIndpTimeStretch(bool b)
@@ -303,7 +361,6 @@ void EngineBuffer::slotTrackLoaded(TrackPointer pTrack,
     m_pTrackSampleRate->set(iTrackSampleRate);
     slotControlSeek(0.);
 
-
     // Let the engine know that a track is loaded now.
     m_pTrackEndCOT->slotSet(0.0f); //XXX: Not sure if to use the COT or CO here
 
@@ -319,6 +376,10 @@ void EngineBuffer::slotTrackLoadFailed(TrackPointer pTrack,
     emit(trackLoadFailed(pTrack, reason));
 }
 
+TrackPointer EngineBuffer::getLoadedTrack() const {
+    return m_pCurrentTrack;
+}
+
 void EngineBuffer::ejectTrack() {
     // Don't allow ejections while playing a track. We don't need to lock to
     // call ControlObject::get() so this is fine.
@@ -339,36 +400,26 @@ void EngineBuffer::ejectTrack() {
     emit(trackUnloaded(pTrack));
 }
 
-
 // WARNING: This method runs in both the GUI thread and the Engine Thread
 void EngineBuffer::slotControlSeek(double change)
 {
-    if(isnan(change) || change > 1.0 || change < 0.0) {
+    if(isnan(change) || change > kMaxPlayposRange || change < kMinPlayposRange) {
         // This seek is ridiculous.
         return;
     }
 
     // Find new playpos, restrict to valid ranges.
     double new_playpos = round(change*file_length_old);
+
+    // TODO(XXX) currently not limiting seeks file_length_old instead of
+    // kMaxPlayposRange.
     if (new_playpos > file_length_old)
         new_playpos = file_length_old;
-    if (new_playpos < 0.)
-        new_playpos = 0.;
 
     // Ensure that the file position is even (remember, stereo channel files...)
     if (!even((int)new_playpos))
         new_playpos--;
 
-    // Give EngineControl's a chance to veto or correct the seek target.
-
-    // Seek reader
-    Hint seek_hint;
-    seek_hint.sample = new_playpos;
-    seek_hint.length = 0;
-    seek_hint.priority = 1;
-    QList<Hint> hint_list;
-    hint_list.append(seek_hint);
-    m_pReader->hintAndMaybeWake(hint_list);
     setNewPlaypos(new_playpos);
 }
 
@@ -408,6 +459,14 @@ void EngineBuffer::slotControlPlayFromStart(double v)
     }
 }
 
+void EngineBuffer::slotControlJumpToStartAndStop(double v)
+{
+    if (v > 0.0) {
+        slotControlSeek(0.);
+        playButton->set(0);
+    }
+}
+
 void EngineBuffer::slotControlStop(double v)
 {
     if (v > 0.0) {
@@ -417,7 +476,8 @@ void EngineBuffer::slotControlStop(double v)
 
 void EngineBuffer::process(const CSAMPLE *, const CSAMPLE * pOut, const int iBufferSize)
 {
-
+    Q_ASSERT(even(iBufferSize));
+    m_pReader->process();
     // Steps:
     // - Lookup new reader information
     // - Calculate current rate
@@ -429,17 +489,10 @@ void EngineBuffer::process(const CSAMPLE *, const CSAMPLE * pOut, const int iBuf
     //   miscellaneous upkeep issues.
 
     CSAMPLE * pOutput = (CSAMPLE *)pOut;
-
     bool bCurBufferPaused = false;
+    double rate = 0;
 
     if (!m_pTrackEnd->get() && pause.tryLock()) {
-
-        if (m_pKeylock->get() && m_pScale != m_pScaleST) {
-            setPitchIndpTimeStretch(true);
-        } else if (!m_pKeylock->get() && m_pScale == m_pScaleST) {
-            setPitchIndpTimeStretch(false);
-        }
-
         float sr = m_pSampleRate->get();
 
         double baserate = 0.0f;
@@ -448,19 +501,36 @@ void EngineBuffer::process(const CSAMPLE *, const CSAMPLE * pOut, const int iBuf
 
         bool paused = playButton->get() != 0.0f ? false : true;
 
-        double rate = m_pRateControl->calculateRate(baserate, paused);
+        bool is_scratching = false;
+        rate = m_pRateControl->calculateRate(baserate, paused, iBufferSize,
+                                             &is_scratching);
         //qDebug() << "rate" << rate << " paused" << paused;
 
+        // Scratching always disables keylock because keylock sounds terrible
+        // when not going at a constant rate.
+        if (is_scratching && m_pScale != m_pScaleLinear) {
+            setPitchIndpTimeStretch(false);
+        } else if (!is_scratching) {
+            if (m_pKeylock->get() && m_pScale != m_pScaleST) {
+                setPitchIndpTimeStretch(true);
+            } else if (!m_pKeylock->get() && m_pScale == m_pScaleST) {
+                setPitchIndpTimeStretch(false);
+            }
+        }
+
         // If the rate has changed, set it in the scale object
         if (rate != rate_old || m_bScalerChanged) {
             // The rate returned by the scale object can be different from the wanted rate!
-
-            //XXX: Trying to force RAMAN to read from correct
-            //     playpos when rate changes direction - Albert
-            if (m_bScalerChanged || (rate_old <= 0 && rate > 0) ||
-                (rate_old >= 0 && rate < 0))
-            {
+            // Make sure new scaler has proper position
+            if (m_bScalerChanged) {
                 setNewPlaypos(filepos_play);
+            } else if (m_pScale != m_pScaleLinear) { //linear scaler does this part for us now
+                //XXX: Trying to force RAMAN to read from correct
+                //     playpos when rate changes direction - Albert
+                if ((rate_old <= 0 && rate > 0) ||
+                    (rate_old >= 0 && rate < 0)) {
+                    setNewPlaypos(filepos_play);
+                }
             }
 
             rate_old = rate;
@@ -479,83 +549,88 @@ void EngineBuffer::process(const CSAMPLE *, const CSAMPLE * pOut, const int iBuf
         // If we're playing past the end, playing before the start, or standing
         // still then by definition the buffer is paused.
         bCurBufferPaused = rate == 0 ||
-            (at_start && backwards) ||
+            //(at_start && backwards) ||
             (at_end && !backwards);
 
+
         // If the buffer is not paused, then scale the audio.
         if (!bCurBufferPaused) {
             CSAMPLE *output;
-            double idx;
-
-            Q_ASSERT(even(iBufferSize));
 
             // The fileposition should be: (why is this thing a double anyway!?
             // Integer valued.
-            Q_ASSERT(round(filepos_play) == filepos_play);
+            double filepos_play_rounded = round(filepos_play);
+            if (filepos_play_rounded != filepos_play) {
+                qWarning() << __FILE__ << __LINE__ << "ERROR: filepos_play is not round:" << filepos_play;
+                filepos_play = filepos_play_rounded;
+            }
+
             // Even.
-            Q_ASSERT(even(filepos_play));
+            if (!even(filepos_play)) {
+                qWarning() << "ERROR: filepos_play is not even:" << filepos_play;
+                filepos_play--;
+            }
 
             // Perform scaling of Reader buffer into buffer.
             output = m_pScale->scale(0,
                                      iBufferSize,
                                      0,
                                      0);
-            idx = m_pScale->getNewPlaypos();
+            double samplesRead = m_pScale->getNewPlaypos();
 
             // qDebug() << "sourceSamples used " << iSourceSamples
-            //          <<" idx " << idx
+            //          <<" samplesRead " << samplesRead
             //          << ", buffer pos " << iBufferStartSample
             //          << ", play " << filepos_play
             //          << " bufferlen " << iBufferSize;
 
             // Copy scaled audio into pOutput
-            memcpy(pOutput, output, sizeof(CSAMPLE) * iBufferSize);
-
-            // for(int i=0; i<iBufferSize; i++) {
-            //     pOutput[i] = output[i];
-            // }
-
-            // Adjust filepos_play by the amount we processed.
-            filepos_play += idx;
-            filepos_play = math_max(0, filepos_play);
-            // We need the above protection against negative playpositions
-            // in case SoundTouch/EngineBufferSoundTouch gives us too many samples.
-
-            // Get rid of annoying decimals that the scaler sometimes produces
-            filepos_play = round(filepos_play);
-
-            if (!even(filepos_play))
-                filepos_play--;
+            memcpy(pOutput, output, sizeof(pOutput[0]) * iBufferSize);
 
+            // Adjust filepos_play by the amount we processed. TODO(XXX) what
+            // happens if samplesRead is a fraction?
+            filepos_play =
+                    m_pReadAheadManager->getEffectiveVirtualPlaypositionFromLog(
+                        static_cast<int>(filepos_play), samplesRead);
         } // else (bCurBufferPaused)
 
         m_engineLock.lock();
         QListIterator<EngineControl*> it(m_engineControls);
         while (it.hasNext()) {
             EngineControl* pControl = it.next();
-            pControl->setCurrentSample(filepos_play);
+            pControl->setCurrentSample(filepos_play, file_length_old);
             double control_seek = pControl->process(rate, filepos_play,
                                                     file_length_old, iBufferSize);
 
             if (control_seek != kNoTrigger) {
+                // If we have not processed loops by this point then we have a
+                // bug. RAMAN should be in charge of taking loops now. This
+                // final step is more to notify all the EngineControls of the
+                // happenings of the engine. TODO(rryan) log condition to a
+                // stats-pipe once we have them.
+
                 filepos_play = control_seek;
-                Q_ASSERT(round(filepos_play) == filepos_play);
+                double filepos_play_rounded = round(filepos_play);
+                if (filepos_play_rounded != filepos_play) {
+                    qWarning() << __FILE__ << __LINE__ << "ERROR: filepos_play is not round:" << filepos_play;
+                    filepos_play = filepos_play_rounded;
+                }
+
+                // Fix filepos_play so that it is not out of bounds.
+                if (file_length_old > 0) {
+                    if (filepos_play > file_length_old) {
+                        // TODO(XXX) limit to kMaxPlayposRange instead of file_length_old
+                        filepos_play = file_length_old;
+                    } else if(filepos_play < file_length_old * kMinPlayposRange) {
+                        filepos_play = kMinPlayposRange * file_length_old;
+                    }
+                }
 
                 // Safety check that the EngineControl didn't pass us a bogus
                 // value
                 if (!even(filepos_play))
                     filepos_play--;
 
-                // Fix filepos_play so that it is not out of bounds.
-                if (file_length_old > 0) {
-                    if(filepos_play > file_length_old) {
-                        filepos_play = file_length_old;
-                        at_end = true;
-                    } else if(filepos_play < 0) {
-                        filepos_play = 0;
-                        at_start = true;
-                    }
-                }
                 // TODO(XXX) need to re-evaluate this later. If we
                 // setNewPlaypos, that clear()'s soundtouch, which might screw
                 // up the audio. This sort of jump is a normal event. Also, the
@@ -566,15 +641,15 @@ void EngineBuffer::process(const CSAMPLE *, const CSAMPLE * pOut, const int iBuf
 
                 //setNewPlaypos(filepos_play);
                 m_pReadAheadManager->notifySeek(filepos_play);
+                // Notify seek the rate control since it needs to track things
+                // like looping. Hacky, I know, but this helps prevent things
+                // like the scratch controller from flipping out.
+                m_pRateControl->notifySeek(filepos_play);
             }
         }
         m_engineLock.unlock();
 
 
-        // Give the Reader hints as to which chunks of the current song we
-        // really care about. It will try very hard to keep these in memory
-        hintReader(rate, iBufferSize);
-
         // Update all the indicators that EngineBuffer publishes to allow
         // external parts of Mixxx to observe its status.
         updateIndicators(rate, iBufferSize);
@@ -585,7 +660,7 @@ void EngineBuffer::process(const CSAMPLE *, const CSAMPLE * pOut, const int iBuf
 
         bool repeat_enabled = m_pRepeat->get() != 0.0f;
 
-        bool end_of_track = (at_start && backwards) ||
+        bool end_of_track = //(at_start && backwards) ||
             (at_end && !backwards);
 
         // If playbutton is pressed, check if we are at start or end of track
@@ -606,61 +681,65 @@ void EngineBuffer::process(const CSAMPLE *, const CSAMPLE * pOut, const int iBuf
         bCurBufferPaused = true;
     }
 
-    // Wake up the reader so that it processes our hints / loads new files
-    // (hopefully) before the next callback.
-    m_pReader->wake();
+    // Give the Reader hints as to which chunks of the current song we
+    // really care about. It will try very hard to keep these in memory
+    hintReader(rate, iBufferSize);
 
-    // Force ramp in if this is the first buffer during a play
     if (m_bLastBufferPaused && !bCurBufferPaused) {
-        // Ramp from zero
-        int iLen = math_min(iBufferSize, kiRampLength);
-        float fStep = pOutput[iLen-1]/(float)iLen;
-        for (int i=0; i<iLen; ++i)
-            pOutput[i] = fStep*i;
+        if (fabs(rate) > 0.005) //at very slow forward rates, don't ramp up
+            m_iRampState = ENGINE_RAMP_UP;
+    } else if (!m_bLastBufferPaused && bCurBufferPaused) {
+        m_iRampState = ENGINE_RAMP_DOWN;
+    } else { //we are not changing state
+        //make sure we aren't accidentally ramping down
+        //this is how we make sure that ramp value will become 1.0 eventually
+        if (fabs(rate) > 0.005 && m_iRampState != ENGINE_RAMP_UP && m_fRampValue < 1.0)
+            m_iRampState = ENGINE_RAMP_UP;
+    }
 
+    //let's try holding the last sample value constant, and pull it
+    //towards zero
+    float ramp_inc = 0;
+    if (m_iRampState == ENGINE_RAMP_UP ||
+        m_iRampState == ENGINE_RAMP_DOWN) {
+        ramp_inc = m_iRampState * 300 / m_pSampleRate->get();
+
+        for (int i=0; i<iBufferSize; i+=2) {
+            if (bCurBufferPaused) {
+                float dither = m_pDitherBuffer[m_iDitherBufferReadIndex];
+                m_iDitherBufferReadIndex = (m_iDitherBufferReadIndex + 1) % MAX_BUFFER_LEN;
+                pOutput[i] = m_fLastSampleValue[0] * m_fRampValue + dither;
+                pOutput[i+1] = m_fLastSampleValue[1] * m_fRampValue + dither;
+            } else {
+                pOutput[i] = pOutput[i] * m_fRampValue;
+                pOutput[i+1] = pOutput[i+1] * m_fRampValue;
+            }
+
+            m_fRampValue += ramp_inc;
+            if (m_fRampValue >= 1.0) {
+                m_iRampState = ENGINE_RAMP_NONE;
+                m_fRampValue = 1.0;
+            }
+            if (m_fRampValue <= 0.0) {
+                m_iRampState = ENGINE_RAMP_NONE;
+                m_fRampValue = 0.0;
+            }
+        }
+    } else if (m_fRampValue == 0.0) {
+        SampleUtil::applyGain(pOutput, 0.0, iBufferSize);
     }
-    // Force ramp out if this is the first buffer to be paused.
-    else if (!m_bLastBufferPaused && bCurBufferPaused) {
-        rampOut(pOut, iBufferSize);
-    }
-    // If the previous buffer was paused and we are currently paused, then
-    // memset the output to zero.
-    else if (m_bLastBufferPaused && bCurBufferPaused) {
-        memset(pOutput, 0, sizeof(pOutput[0])*iBufferSize);
+
+    if ((!bCurBufferPaused && m_iRampState == ENGINE_RAMP_NONE) ||
+        (bCurBufferPaused && m_fRampValue == 0.0)) {
+        m_fLastSampleValue[0] = pOutput[iBufferSize-2];
+        m_fLastSampleValue[1] = pOutput[iBufferSize-1];
     }
 
+    /*for (int i=0; i<iBufferSize; i+=2) {
+        writer << pOutput[i] <<  "\n";
+    }*/
+
     m_bLastBufferPaused = bCurBufferPaused;
-    m_fLastSampleValue = pOutput[iBufferSize-1];
-}
-
-
-void EngineBuffer::rampOut(const CSAMPLE* pOut, int iBufferSize)
-{
-    CSAMPLE * pOutput = (CSAMPLE *)pOut;
-
-    //qDebug() << "ramp out";
-
-    // Ramp to zero
-    int i=0;
-    if (m_fLastSampleValue!=0.)
-    {
-        int iLen = math_min(iBufferSize, kiRampLength);
-        float fStep = m_fLastSampleValue/(float)iLen;
-        // TODO(XXX) SSE
-        while (i<iLen)
-        {
-            pOutput[i] = fStep*(iLen-(i+1));
-            ++i;
-        }
-    }
-
-    // TODO(XXX) memset
-    // Reset rest of buffer
-    while (i<iBufferSize)
-    {
-        pOutput[i]=0.;
-        ++i;
-    }
 }
 
 void EngineBuffer::updateIndicators(double rate, int iBufferSize) {
@@ -670,21 +749,27 @@ void EngineBuffer::updateIndicators(double rate, int iBufferSize) {
 
     double fFractionalPlaypos = 0.0;
     if (file_length_old!=0.) {
-        fFractionalPlaypos = math_max(0.,math_min(filepos_play,file_length_old));
+        fFractionalPlaypos = math_min(filepos_play,file_length_old);
         fFractionalPlaypos /= file_length_old;
     } else {
         fFractionalPlaypos = 0.;
     }
 
     // Update indicators that are only updated after every
-    // sampleRate/UPDATE_RATE samples processed.  (e.g. playposSlider,
+    // sampleRate/kiUpdateRate samples processed.  (e.g. playposSlider,
     // rateEngine)
-    if (m_iSamplesCalculated > (m_pSampleRate->get()/UPDATE_RATE)) {
+    if (m_iSamplesCalculated > (m_pSampleRate->get()/kiUpdateRate)) {
         playposSlider->set(fFractionalPlaypos);
 
         if(rate != rateEngine->get())
             rateEngine->set(rate);
 
+        //Update the BPM even more slowly
+        m_iUiSlowTick = (m_iUiSlowTick + 1) % kiBpmUpdateRate;
+        if (m_iUiSlowTick == 0) {
+            visualBpm->set(m_pBpmControl->getBpm());
+        }
+
         // Reset sample counter
         m_iSamplesCalculated = 0;
     }
@@ -699,7 +784,7 @@ void EngineBuffer::hintReader(const double dRate,
     m_engineLock.lock();
 
     m_hintList.clear();
-    m_pReadAheadManager->hintReader(m_hintList, iSourceSamples);
+    m_pReadAheadManager->hintReader(dRate, m_hintList, iSourceSamples);
 
     QListIterator<EngineControl*> it(m_engineControls);
     while (it.hasNext()) {
@@ -736,6 +821,12 @@ void EngineBuffer::addControl(EngineControl* pControl) {
     connect(pControl, SIGNAL(seekAbs(double)),
             this, SLOT(slotControlSeekAbs(double)),
             Qt::DirectConnection);
+    connect(this, SIGNAL(trackLoaded(TrackPointer)),
+            pControl, SLOT(trackLoaded(TrackPointer)),
+            Qt::DirectConnection);
+    connect(this, SIGNAL(trackUnloaded(TrackPointer)),
+            pControl, SLOT(trackUnloaded(TrackPointer)),
+            Qt::DirectConnection);
 }
 
 void EngineBuffer::bindWorkers(EngineWorkerScheduler* pWorkerScheduler) {
