@@ -27,9 +27,9 @@
 #include "playerinfo.h"
 #include "trackinfoobject.h"
 #ifdef __WINDOWS__
-	#include <windows.h>
-	//sleep on linux assumes seconds where as Sleep on Windows assumes milliseconds
-	#define sleep(x) Sleep(x*1000)
+    #include <windows.h>
+    //sleep on linux assumes seconds where as Sleep on Windows assumes milliseconds
+    #define sleep(x) Sleep(x*1000)
 #endif
 
 #define TIMEOUT 10
@@ -51,9 +51,6 @@ EngineShoutcast::EngineShoutcast(ConfigObject<ConfigValue> *_config)
           m_pUpdateShoutcastFromPrefs(NULL),
           m_pMasterSamplerate(new ControlObjectThread(
               ControlObject::getControl(ConfigKey("[Master]", "samplerate")))),
-          m_pCrossfader(NULL),
-          m_pVolume1(NULL),
-          m_pVolume2(NULL),
           m_shoutMutex(QMutex::Recursive) {
 
     m_pShout = 0;
@@ -63,12 +60,6 @@ EngineShoutcast::EngineShoutcast(ConfigObject<ConfigValue> *_config)
 
     m_bQuit = false;
 
-    m_pCrossfader = new ControlObjectThread(ControlObject::getControl(ConfigKey("[Master]","crossfader")));
-    m_pVolume1 = new ControlObjectThread(ControlObject::getControl(ConfigKey("[Channel1]","volume")));
-    m_pVolume2 = new ControlObjectThread(ControlObject::getControl(ConfigKey("[Channel2]","volume")));
-
-
-
     m_firstCall = false;
     // Initialize libshout
     shout_init();
@@ -103,9 +94,6 @@ EngineShoutcast::~EngineShoutcast()
     delete m_pUpdateShoutcastFromPrefs;
     delete m_pShoutcastNeedUpdateFromPrefs;
     delete m_pMasterSamplerate;
-    delete m_pCrossfader;
-    delete m_pVolume1;
-    delete m_pVolume2;
 
     if (m_pShoutMetaData)
         shout_metadata_free(m_pShoutMetaData);
@@ -120,27 +108,27 @@ bool EngineShoutcast::serverDisconnect()
 {
     QMutexLocker locker(&m_shoutMutex);
     if (m_encoder){
-		m_encoder->flush();
-		delete m_encoder;
-		m_encoder = NULL;
-	}
+        m_encoder->flush();
+        delete m_encoder;
+        m_encoder = NULL;
+    }
 
     if (m_pShout) {
         shout_close(m_pShout);
-		return true;
+        return true;
     }
-	return false; //if no connection has been established, nothing can be disconnected
+    return false; //if no connection has been established, nothing can be disconnected
 }
 
 bool EngineShoutcast::isConnected()
 {
     QMutexLocker locker(&m_shoutMutex);
-	if (m_pShout) {
-		m_iShoutStatus = shout_get_connected(m_pShout);
-    	if (m_iShoutStatus == SHOUTERR_CONNECTED)
-    		return true;
-	}
-	return false;
+    if (m_pShout) {
+        m_iShoutStatus = shout_get_connected(m_pShout);
+        if (m_iShoutStatus == SHOUTERR_CONNECTED)
+            return true;
+    }
+    return false;
 }
 /*
  * Update EngineShoutcast values from the preferences.
@@ -166,11 +154,11 @@ void EngineShoutcast::updateFromPreferences()
     QByteArray baStreamPublic = m_pConfig->getValueString(ConfigKey(SHOUTCAST_PREF_KEY,"stream_public")).toLatin1();
     QByteArray baBitrate    = m_pConfig->getValueString(ConfigKey(SHOUTCAST_PREF_KEY,"bitrate")).toLatin1();
 
-	m_baFormat    = m_pConfig->getValueString(ConfigKey(SHOUTCAST_PREF_KEY,"format")).toLatin1();
+    m_baFormat    = m_pConfig->getValueString(ConfigKey(SHOUTCAST_PREF_KEY,"format")).toLatin1();
 
-	m_custom_metadata = (bool)m_pConfig->getValueString(ConfigKey(SHOUTCAST_PREF_KEY,"enable_metadata")).toInt();
-	m_baCustom_title = m_pConfig->getValueString(ConfigKey(SHOUTCAST_PREF_KEY,"custom_title")).toLatin1();
-	m_baCustom_artist = m_pConfig->getValueString(ConfigKey(SHOUTCAST_PREF_KEY,"custom_artist")).toLatin1();
+    m_custom_metadata = (bool)m_pConfig->getValueString(ConfigKey(SHOUTCAST_PREF_KEY,"enable_metadata")).toInt();
+    m_baCustom_title = m_pConfig->getValueString(ConfigKey(SHOUTCAST_PREF_KEY,"custom_title")).toLatin1();
+    m_baCustom_artist = m_pConfig->getValueString(ConfigKey(SHOUTCAST_PREF_KEY,"custom_artist")).toLatin1();
 
     int format;
     int len;
@@ -178,22 +166,22 @@ void EngineShoutcast::updateFromPreferences()
 
 
     if (shout_set_host(m_pShout, baHost.data()) != SHOUTERR_SUCCESS) {
-		errorDialog("Error setting hostname!", shout_get_error(m_pShout));
+        errorDialog("Error setting hostname!", shout_get_error(m_pShout));
         return;
     }
 
     if (shout_set_protocol(m_pShout, SHOUT_PROTOCOL_HTTP) != SHOUTERR_SUCCESS) {
-		errorDialog("Error setting protocol!", shout_get_error(m_pShout));
+        errorDialog("Error setting protocol!", shout_get_error(m_pShout));
         return;
     }
 
     if (shout_set_port(m_pShout, baPort.toUInt()) != SHOUTERR_SUCCESS) {
-		errorDialog("Error setting port!", shout_get_error(m_pShout));
+        errorDialog("Error setting port!", shout_get_error(m_pShout));
         return;
     }
 
     if (shout_set_password(m_pShout, baPassword.data()) != SHOUTERR_SUCCESS) {
-		errorDialog("Error setting password!", shout_get_error(m_pShout));
+        errorDialog("Error setting password!", shout_get_error(m_pShout));
         return;
     }
     if (shout_set_mount(m_pShout, baMountPoint.data()) != SHOUTERR_SUCCESS) {
@@ -202,22 +190,22 @@ void EngineShoutcast::updateFromPreferences()
     }
 
     if (shout_set_user(m_pShout, baLogin.data()) != SHOUTERR_SUCCESS) {
-		errorDialog("Error setting username!", shout_get_error(m_pShout));
+        errorDialog("Error setting username!", shout_get_error(m_pShout));
         return;
     }
-	if (shout_set_name(m_pShout, baStreamName.data()) != SHOUTERR_SUCCESS) {
-		errorDialog("Error setting stream name!", shout_get_error(m_pShout));
+    if (shout_set_name(m_pShout, baStreamName.data()) != SHOUTERR_SUCCESS) {
+        errorDialog("Error setting stream name!", shout_get_error(m_pShout));
         return;
     }
-	if (shout_set_description(m_pShout, baStreamDesc.data()) != SHOUTERR_SUCCESS) {
-		errorDialog("Error setting stream description!", shout_get_error(m_pShout));
+    if (shout_set_description(m_pShout, baStreamDesc.data()) != SHOUTERR_SUCCESS) {
+        errorDialog("Error setting stream description!", shout_get_error(m_pShout));
         return;
     }
-	if (shout_set_genre(m_pShout, baStreamGenre.data()) != SHOUTERR_SUCCESS) {
-      	errorDialog("Error setting stream genre!", shout_get_error(m_pShout));
+    if (shout_set_genre(m_pShout, baStreamGenre.data()) != SHOUTERR_SUCCESS) {
+          errorDialog("Error setting stream genre!", shout_get_error(m_pShout));
         return;
     }
-	if (shout_set_url(m_pShout, baStreamWebsite.data()) != SHOUTERR_SUCCESS) {
+    if (shout_set_url(m_pShout, baStreamWebsite.data()) != SHOUTERR_SUCCESS) {
        errorDialog("Error setting stream url!", shout_get_error(m_pShout));
        return;
     }
@@ -266,24 +254,24 @@ void EngineShoutcast::updateFromPreferences()
     } else if ( ! qstricmp(baServerType.data(), "Icecast 1")) {
         protocol = SHOUT_PROTOCOL_XAUDIOCAST;
     } else {
-		errorDialog("Error: unknown server protocol!", shout_get_error(m_pShout));
+        errorDialog("Error: unknown server protocol!", shout_get_error(m_pShout));
         return;
     }
 
     if (( protocol == SHOUT_PROTOCOL_ICY ) && ( format != SHOUT_FORMAT_MP3)) {
-		errorDialog("Error: libshout only supports Shoutcast with MP3 format!", shout_get_error(m_pShout));
-		return;
+        errorDialog("Error: libshout only supports Shoutcast with MP3 format!", shout_get_error(m_pShout));
+        return;
     }
 
     if (shout_set_protocol(m_pShout, protocol) != SHOUTERR_SUCCESS) {
-		errorDialog("Error setting protocol!", shout_get_error(m_pShout));
+        errorDialog("Error setting protocol!", shout_get_error(m_pShout));
         return;
     }
 
     // Initialize m_encoder
-	if(m_encoder) {
-		delete m_encoder;		//delete m_encoder if it has been initalized (with maybe) different bitrate
-	}
+    if(m_encoder) {
+        delete m_encoder;        //delete m_encoder if it has been initalized (with maybe) different bitrate
+    }
     if ( ! qstrcmp(m_baFormat, "MP3")) {
         m_encoder = new EncoderMp3(this);
 
@@ -296,11 +284,11 @@ void EngineShoutcast::updateFromPreferences()
         return;
     }
     if (m_encoder->initEncoder(iBitrate) < 0) {
-		//e.g., if lame is not found
-		//init m_encoder itself will display a message box
-		qDebug() << "**** Encoder init failed";
-		delete m_encoder;
-		m_encoder = NULL;
+        //e.g., if lame is not found
+        //init m_encoder itself will display a message box
+        qDebug() << "**** Encoder init failed";
+        delete m_encoder;
+        m_encoder = NULL;
     }
 
 }
@@ -319,19 +307,19 @@ bool EngineShoutcast::serverConnect()
     m_iShoutFailures = 0;
     // set to a high number to automatically update the metadata
     // on the first change
-    m_pMetaDataLife = 31337;
-	//If static metadata is available, we only need to send metadata one time
-	m_firstCall = false;
+    m_iMetaDataLife = 31337;
+    //If static metadata is available, we only need to send metadata one time
+    m_firstCall = false;
 
-	/*Check if m_encoder is initalized
-	 * Encoder is initalized in updateFromPreferences which is called always before serverConnect()
-	 * If m_encoder is NULL, then we propably want to use MP3 streaming, however, lame could not be found
-	 * It does not make sense to connect
-	 */
-	 if(m_encoder == NULL){
-		m_pConfig->set(ConfigKey("[Shoutcast]","enabled"),ConfigValue("0"));
-		return false;
-	}
+    /*Check if m_encoder is initalized
+     * Encoder is initalized in updateFromPreferences which is called always before serverConnect()
+     * If m_encoder is NULL, then we propably want to use MP3 streaming, however, lame could not be found
+     * It does not make sense to connect
+     */
+     if(m_encoder == NULL){
+        m_pConfig->set(ConfigKey("[Shoutcast]","enabled"),ConfigValue("0"));
+        return false;
+    }
     const int iMaxTries = 3;
     while (!m_bQuit && m_iShoutFailures < iMaxTries) {
         if (m_pShout)
@@ -353,8 +341,7 @@ bool EngineShoutcast::serverConnect()
     if (m_iShoutFailures == iMaxTries) {
         if (m_pShout)
             shout_close(m_pShout);
-		m_pConfig->set(ConfigKey("[Shoutcast]","enabled"),ConfigValue("0"));
-
+        m_pConfig->set(ConfigKey("[Shoutcast]","enabled"),ConfigValue("0"));
     }
     if (m_bQuit) {
         if (m_pShout)
@@ -363,23 +350,23 @@ bool EngineShoutcast::serverConnect()
     }
 
     m_iShoutFailures = 0;
-	int timeout = 0;
+    int timeout = 0;
     while (m_iShoutStatus == SHOUTERR_BUSY && timeout < TIMEOUT) {
         qDebug() << "Connection pending. Sleeping...";
         sleep(1);
         m_iShoutStatus = shout_get_connected(m_pShout);
-		++ timeout;
+        ++ timeout;
     }
     if (m_iShoutStatus == SHOUTERR_CONNECTED) {
         qDebug() << "***********Connected to Shoutcast server...";
         return true;
     }
-	//otherwise disable shoutcast in preferences
-	m_pConfig->set(ConfigKey("[Shoutcast]","enabled"),ConfigValue("0"));
-	if(m_pShout){
-		shout_close(m_pShout);
-		//errorDialog(tr("Mixxx could not connect to the server"), tr("Please check your connection to the Internet and verify that your username and password are correct."));
-	}
+    //otherwise disable shoutcast in preferences
+    m_pConfig->set(ConfigKey("[Shoutcast]","enabled"),ConfigValue("0"));
+    if(m_pShout){
+        shout_close(m_pShout);
+        //errorDialog(tr("Mixxx could not connect to the server"), tr("Please check your connection to the Internet and verify that your username and password are correct."));
+    }
 
     return false;
 }
@@ -446,111 +433,60 @@ void EngineShoutcast::write(unsigned char *header, unsigned char *body,
 void EngineShoutcast::process(const CSAMPLE *, const CSAMPLE *pOut, const int iBufferSize)
 {
     QMutexLocker locker(&m_shoutMutex);
-	 //Check to see if Shoutcast is enabled, and pass the samples off to be broadcast if necessary.
+     //Check to see if Shoutcast is enabled, and pass the samples off to be broadcast if necessary.
      bool prefEnabled = (m_pConfig->getValueString(ConfigKey("[Shoutcast]","enabled")).toInt() == 1);
 
     if (prefEnabled) {
-	    if(!isConnected()){
-			//Initialize the m_pShout structure with the info from Mixxx's m_shoutcast preferences.
-			updateFromPreferences();
+        if(!isConnected()){
+            //Initialize the m_pShout structure with the info from Mixxx's m_shoutcast preferences.
+            updateFromPreferences();
 
-			if(serverConnect()){
-				ErrorDialogProperties* props = ErrorDialogHandler::instance()->newDialogProperties();
-				props->setType(DLG_INFO);
-				props->setTitle(tr("Live broadcasting"));
-				props->setText(tr("Mixxx has successfully connected to the shoutcast server"));
-				ErrorDialogHandler::instance()->requestErrorDialog(props);
-			}
+            if(serverConnect()){
+                ErrorDialogProperties* props = ErrorDialogHandler::instance()->newDialogProperties();
+                props->setType(DLG_INFO);
+                props->setTitle(tr("Live broadcasting"));
+                props->setText(tr("Mixxx has successfully connected to the shoutcast server"));
+                ErrorDialogHandler::instance()->requestErrorDialog(props);
+            }
             else{
                 errorDialog(tr("Mixxx could not connect to streaming server"), tr("Please check your connection to the Internet and verify that your username and password are correct."));
 
             }
-		}
+        }
         //send to shoutcast, if connection has been established
         if (m_iShoutStatus != SHOUTERR_CONNECTED)
             return;
 
         if (iBufferSize > 0 && m_encoder){
             m_encoder->encodeBuffer(pOut, iBufferSize); //encode and send to shoutcast
-	    }
+        }
         //Check if track has changed and submit its new metadata to shoutcast
         if (metaDataHasChanged())
             updateMetaData();
 
         if (m_pUpdateShoutcastFromPrefs->get() > 0.0f){
-	        /*
-			 * You cannot change bitrate, hostname, etc while connected to a stream
-		     */
-			serverDisconnect();
-			updateFromPreferences();
-			serverConnect();
-		}
+            /*
+             * You cannot change bitrate, hostname, etc while connected to a stream
+             */
+            serverDisconnect();
+            updateFromPreferences();
+            serverConnect();
+        }
      }
     //if shoutcast is disabled
-	else{
-		if(isConnected()){
-			serverDisconnect();
-			ErrorDialogProperties* props = ErrorDialogHandler::instance()->newDialogProperties();
-			props->setType(DLG_INFO);
-			props->setTitle(tr("Live broadcasting"));
-			props->setText(tr("Mixxx has successfully disconnected to the shoutcast server"));
+    else{
+        if(isConnected()){
+            serverDisconnect();
+            ErrorDialogProperties* props = ErrorDialogHandler::instance()->newDialogProperties();
+            props->setType(DLG_INFO);
+            props->setTitle(tr("Live broadcasting"));
+            props->setText(tr("Mixxx has successfully disconnected to the shoutcast server"));
 
-			ErrorDialogHandler::instance()->requestErrorDialog(props);
-		}
-	}
-}
-
-/*
- * Algorithm which simply flips the lowest and/or second lowest bits,
- * bits 1 and 2, to represent which track is active and returns the result.
- */
-int EngineShoutcast::getActiveTracks()
-{
-    int tracks = 0;
-
-
-    if (ControlObject::getControl(ConfigKey("[Channel1]","play"))->get()==1.) tracks |= 1;
-    if (ControlObject::getControl(ConfigKey("[Channel2]","play"))->get()==1.) tracks |= 2;
-
-    if (tracks ==  0)
-        return 0;
-
-    // Detect the dominant track by checking the crossfader and volume levels
-    if ((tracks & 1) && (tracks & 2)) {
-
-        if ((m_pVolume1->get() == 0) && (m_pVolume2->get() == 0))
-            return 0;
-
-        if (m_pVolume2->get() == 0) {
-            tracks = 1;
+            ErrorDialogHandler::instance()->requestErrorDialog(props);
         }
-        else if ( m_pVolume1->get() == 0) {
-            tracks = 2;
-        }
-        // allow a bit of leeway with the crossfader
-        else if ((m_pCrossfader->get() < 0.05) && (m_pCrossfader->get() > -0.05)) {
-
-            if (m_pVolume1->get() > m_pVolume2->get()) {
-                tracks = 1;
-            }
-            else if (m_pVolume1->get() < m_pVolume2->get()) {
-                tracks = 2;
-            }
-
-        }
-        else if ( m_pCrossfader->get() < -0.05 ) {
-            tracks = 1;
-        }
-        else if ( m_pCrossfader->get() > 0.05 ) {
-            tracks = 2;
-        }
-
     }
-
-    return tracks;
 }
 
-
 /*
  * Check if the metadata has changed since the previous check.
  * We also check when was the last check performed to avoid using
@@ -560,50 +496,35 @@ int EngineShoutcast::getActiveTracks()
 bool EngineShoutcast::metaDataHasChanged()
 {
     QMutexLocker locker(&m_shoutMutex);
-    int tracks;
-    TrackPointer newMetaData;
-    bool changed = false;
+    TrackPointer pTrack;
 
 
-    if ( m_pMetaDataLife < 16 ) {
-        m_pMetaDataLife++;
+    if ( m_iMetaDataLife < 16 ) {
+        m_iMetaDataLife++;
         return false;
     }
 
-    m_pMetaDataLife = 0;
+    m_iMetaDataLife = 0;
 
 
-    tracks = getActiveTracks();
+    pTrack = PlayerInfo::Instance().getCurrentPlayingTrack();
+    if ( !pTrack )
+        return false;
 
-    switch (tracks)
-    {
-    case 0:
-        // no tracks are playing
-        // we should set the metadata to nothing
-        break;
-    case 1:
-        // track 1 is active
-        newMetaData = PlayerInfo::Instance().getTrackInfo("[Channel1]");
-        if (newMetaData != m_pMetaData)
-        {
-            m_pMetaData = newMetaData;
-            changed = true;
+    if ( m_pMetaData ) {
+        if ((pTrack->getId() == -1) || (m_pMetaData->getId() == -1)) {
+            if ((pTrack->getArtist() == m_pMetaData->getArtist()) &&
+                (pTrack->getTitle() == m_pMetaData->getArtist())) {
+                return false;
+            }
         }
-        break;
-    case 2:
-        // track 2 is active
-		newMetaData = PlayerInfo::Instance().getTrackInfo("[Channel2]");
-		if (newMetaData != m_pMetaData)
-        {
-            m_pMetaData = newMetaData;
-            changed = true;
+        else if (pTrack->getId() == m_pMetaData->getId()) {
+            return false;
         }
-        break;
-    case 3:
-        // both tracks are active, just stick with it for now
-        break;
     }
-    return changed;
+
+    m_pMetaData = pTrack;
+    return true;
 }
 
 /*
@@ -618,51 +539,48 @@ void EngineShoutcast::updateMetaData()
         return;
 
     QByteArray baSong = "";
-	/**
+    /**
      * If track has changed and static metadata is disabled
-	 * Send new metadata to shoutcast!
+     * Send new metadata to shoutcast!
      * This works only for MP3 streams properly as stated in comments, see shout.h
-	 * WARNING: Changing OGG metadata dynamically by using shout_set_metadata
-	 * will cause stream interruptions to listeners
-	 *
-	 * Also note: Do not try to include Vorbis comments in OGG packages and send them to stream.
-	 * This was done in EncoderVorbis previously and caused interruptions on track change as well
-	 * which sounds awful to listeners.
-
-	 * To conlcude: Only write OGG metadata one time, i.e., if static metadata is used.
- 	 */
+     * WARNING: Changing OGG metadata dynamically by using shout_set_metadata
+     * will cause stream interruptions to listeners
+     *
+     * Also note: Do not try to include Vorbis comments in OGG packages and send them to stream.
+     * This was done in EncoderVorbis previously and caused interruptions on track change as well
+     * which sounds awful to listeners.
+     * To conlcude: Only write OGG metadata one time, i.e., if static metadata is used.
+      */
 
 
     //If we use MP3 streaming and want dynamic metadata changes
-	if(!m_custom_metadata && !qstrcmp(m_baFormat, "MP3")){
-		if (m_pMetaData != NULL) {
-		    // convert QStrings to char*s
-		    QByteArray baArtist = m_pMetaData->getArtist().toLatin1();
-		    QByteArray baTitle = m_pMetaData->getTitle().toLatin1();
+    if(!m_custom_metadata && !qstrcmp(m_baFormat, "MP3")){
+        if (m_pMetaData != NULL) {
+            // convert QStrings to char*s
+            QByteArray baArtist = m_pMetaData->getArtist().toLatin1();
+            QByteArray baTitle = m_pMetaData->getTitle().toLatin1();
 
-		    if (baArtist.isEmpty())
-		        baSong = baTitle;
-		    else
-		        baSong = baArtist + " - " + baTitle;
+            if (baArtist.isEmpty())
+                baSong = baTitle;
+            else
+                baSong = baArtist + " - " + baTitle;
 
-			/** Update metadata */
-			shout_metadata_add(m_pShoutMetaData, "song",  baSong.data());
-    		shout_set_metadata(m_pShout, m_pShoutMetaData);
-		}
-	}
+            /** Update metadata */
+            shout_metadata_add(m_pShoutMetaData, "song",  baSong.data());
+                shout_set_metadata(m_pShout, m_pShoutMetaData);
+        }
+    }
     //Otherwise we might use static metadata
-	else{
-		/** If we use static metadata, we only need to call the following line once **/
-		if(m_custom_metadata && !m_firstCall){
-			baSong = m_baCustom_artist + " - " + m_baCustom_title;
-			/** Update metadata */
-			shout_metadata_add(m_pShoutMetaData, "song",  baSong.data());
-    		shout_set_metadata(m_pShout, m_pShoutMetaData);
-			m_firstCall = true;
-		}
-	}
-
-
+    else{
+        /** If we use static metadata, we only need to call the following line once **/
+        if(m_custom_metadata && !m_firstCall){
+            baSong = m_baCustom_artist + " - " + m_baCustom_title;
+            /** Update metadata */
+            shout_metadata_add(m_pShoutMetaData, "song",  baSong.data());
+                shout_set_metadata(m_pShout, m_pShoutMetaData);
+            m_firstCall = true;
+        }
+    }
 }
 /* -------- ------------------------------------------------------
 Purpose: Common error dialog creation code for run-time exceptions
