@@ -14,8 +14,10 @@ const bool sDebug = false;
 
 BaseSqlTableModel::BaseSqlTableModel(QObject* pParent,
                                      TrackCollection* pTrackCollection,
-                                     QSqlDatabase db)
+                                     QSqlDatabase db,
+                                     QString settingsNamespace)
         :  QAbstractTableModel(pParent),
+           TrackModel(db, settingsNamespace),
            m_pTrackCollection(pTrackCollection),
            m_trackDAO(m_pTrackCollection->getTrackDAO()),
            m_database(db) {
@@ -125,7 +127,7 @@ QString BaseSqlTableModel::orderByClause() const {
 
     QString s;
     s.append(QLatin1String("ORDER BY "));
-    QString sort_field = QString("%1.%2").arg(m_tableName).arg(field);
+    QString sort_field = QString("%1.%2").arg(m_tableName, field);
     s.append(sort_field);
 
     s.append((m_eSortOrder == Qt::AscendingOrder) ? QLatin1String(" ASC") :
@@ -168,7 +170,7 @@ void BaseSqlTableModel::select() {
     QString columns = m_tableColumnsJoined;
     QString orderBy = orderByClause();
     QString queryString = QString("SELECT %1 FROM %2 %3")
-            .arg(columns).arg(m_tableName).arg(orderBy);
+            .arg(columns, m_tableName, orderBy);
 
     if (sDebug) {
         qDebug() << this << "select() executing:" << queryString;
@@ -236,18 +238,26 @@ void BaseSqlTableModel::select() {
                                  sortColumn, m_eSortOrder,
                                  &m_trackSortOrder);
 
-    // Only re-sort results if the sort column was a track column.
-    if (sortColumn > 0) {
-        for (QVector<RowInfo>::iterator it = rowInfo.begin();
-             it != rowInfo.end(); ++it) {
+    // Re-sort the track IDs since filterAndSort can change their order or mark
+    // them for removal (by setting their row to -1).
+    for (QVector<RowInfo>::iterator it = rowInfo.begin();
+         it != rowInfo.end(); ++it) {
+        // If the sort column is not a track column then we will sort only to
+        // separate removed tracks (order == -1) from present tracks (order ==
+        // 0). Otherwise we sort by the order that filterAndSort returned to us.
+        if (sortColumn == 0) {
+            it->order = m_trackSortOrder.contains(it->trackId) ? 0 : -1;
+        } else {
             it->order = m_trackSortOrder.value(it->trackId, -1);
         }
-
-        // RowInfo::operator< sorts by the order field, except -1 is placed at the
-        // end so we can easily slice off rows that are no longer present.
-        qSort(rowInfo.begin(), rowInfo.end());
     }
 
+    // RowInfo::operator< sorts by the order field, except -1 is placed at the
+    // end so we can easily slice off rows that are no longer present. Stable
+    // sort is necessary because the tracks may be in pre-sorted order so we
+    // should not disturb that if we are only removing tracks.
+    qStableSort(rowInfo.begin(), rowInfo.end());
+
     m_trackIdToRows.clear();
     for (int i = 0; i < rowInfo.size(); ++i) {
         const RowInfo& row = rowInfo[i];
@@ -303,7 +313,7 @@ void BaseSqlTableModel::setTable(const QString& tableName,
     m_bDirty = true;
 }
 
-QString BaseSqlTableModel::currentSearch() const {
+const QString BaseSqlTableModel::currentSearch() const {
     return m_currentSearch;
 }
 
@@ -412,8 +422,10 @@ QVariant BaseSqlTableModel::data(const QModelIndex& index, int role) const {
         case Qt::ToolTipRole:
         case Qt::DisplayRole:
             if (column == fieldIndex(LIBRARYTABLE_DURATION)) {
-                if (qVariantCanConvert<int>(value))
-                    value = MixxxUtils::secondsToMinutes(qVariantValue<int>(value));
+                if (qVariantCanConvert<int>(value)) {
+                    value = MixxxUtils::secondsToMinutes(
+                        qVariantValue<int>(value));
+                }
             } else if (column == fieldIndex(LIBRARYTABLE_RATING)) {
                 if (qVariantCanConvert<int>(value))
                     value = qVariantFromValue(StarRating(value.toInt()));
@@ -421,8 +433,8 @@ QVariant BaseSqlTableModel::data(const QModelIndex& index, int role) const {
                 if (qVariantCanConvert<int>(value))
                     value =  QString("(%1)").arg(value.toInt());
             } else if (column == fieldIndex(LIBRARYTABLE_PLAYED)) {
-                // Convert to a bool. Not really that useful since it gets converted
-                // right back to a QVariant
+                // Convert to a bool. Not really that useful since it gets
+                // converted right back to a QVariant
                 value = (value == "true") ? true : false;
             } else if (column == fieldIndex(LIBRARYTABLE_DATETIMEADDED)) {
                 value = value.toDateTime();
@@ -432,7 +444,8 @@ QVariant BaseSqlTableModel::data(const QModelIndex& index, int role) const {
             if (column == fieldIndex(LIBRARYTABLE_BPM)) {
                 return value.toDouble();
             } else if (column == fieldIndex(LIBRARYTABLE_TIMESPLAYED)) {
-                return index.sibling(row, fieldIndex(LIBRARYTABLE_PLAYED)).data().toBool();
+                return index.sibling(
+                    row, fieldIndex(LIBRARYTABLE_PLAYED)).data().toBool();
             } else if (column == fieldIndex(LIBRARYTABLE_RATING)) {
                 if (qVariantCanConvert<int>(value))
                     value = qVariantFromValue(StarRating(value.toInt()));
@@ -440,7 +453,8 @@ QVariant BaseSqlTableModel::data(const QModelIndex& index, int role) const {
             break;
         case Qt::CheckStateRole:
             if (column == fieldIndex(LIBRARYTABLE_TIMESPLAYED)) {
-                bool played = index.sibling(row, fieldIndex(LIBRARYTABLE_PLAYED)).data().toBool();
+                bool played = index.sibling(
+                    row, fieldIndex(LIBRARYTABLE_PLAYED)).data().toBool();
                 value = played ? Qt::Checked : Qt::Unchecked;
             }
             break;
@@ -450,7 +464,8 @@ QVariant BaseSqlTableModel::data(const QModelIndex& index, int role) const {
     return value;
 }
 
-bool BaseSqlTableModel::setData(const QModelIndex& index, const QVariant& value, int role) {
+bool BaseSqlTableModel::setData(
+    const QModelIndex& index, const QVariant& value, int role) {
     if (!index.isValid())
         return false;
 
@@ -485,6 +500,8 @@ bool BaseSqlTableModel::setData(const QModelIndex& index, const QVariant& value,
         return false;
     }
 
+    // TODO(rryan) ugly and only works because the mixxx library tables are the
+    // only ones that aren't read-only. This should be moved into BTC.
     TrackPointer pTrack = m_trackDAO.getTrack(trackId);
     setTrackValueForColumn(pTrack, column, value);
 
@@ -500,14 +517,15 @@ Qt::ItemFlags BaseSqlTableModel::flags(const QModelIndex &index) const {
     return readWriteFlags(index);
 }
 
-Qt::ItemFlags BaseSqlTableModel::readWriteFlags(const QModelIndex &index) const {
+Qt::ItemFlags BaseSqlTableModel::readWriteFlags(
+    const QModelIndex &index) const {
     if (!index.isValid())
         return Qt::ItemIsEnabled;
 
     Qt::ItemFlags defaultFlags = QAbstractItemModel::flags(index);
 
-    //Enable dragging songs from this data model to elsewhere (like the waveform
-    //widget to load a track into a Player).
+    // Enable dragging songs from this data model to elsewhere (like the
+    // waveform widget to load a track into a Player).
     defaultFlags |= Qt::ItemIsDragEnabled;
 
     int row = index.row();
@@ -526,8 +544,7 @@ Qt::ItemFlags BaseSqlTableModel::readWriteFlags(const QModelIndex &index) const
     }
 }
 
-Qt::ItemFlags BaseSqlTableModel::readOnlyFlags(const QModelIndex &index) const
-{
+Qt::ItemFlags BaseSqlTableModel::readOnlyFlags(const QModelIndex &index) const {
     Qt::ItemFlags defaultFlags = QAbstractItemModel::flags(index);
     if (!index.isValid())
         return Qt::ItemIsEnabled;
@@ -546,6 +563,22 @@ const QLinkedList<int> BaseSqlTableModel::getTrackRows(int trackId) const {
     return QLinkedList<int>();
 }
 
+int BaseSqlTableModel::getTrackId(const QModelIndex& index) const {
+    if (!index.isValid()) {
+        return -1;
+    }
+    return index.sibling(index.row(), fieldIndex(m_idColumn)).data().toInt();
+}
+
+QString BaseSqlTableModel::getTrackLocation(const QModelIndex& index) const {
+    if (!index.isValid()) {
+        return "";
+    }
+    QString location = index.sibling(
+        index.row(), fieldIndex("location")).data().toString();
+    return location;
+}
+
 void BaseSqlTableModel::tracksChanged(QSet<int> trackIds) {
     if (sDebug) {
         qDebug() << this << "trackChanged" << trackIds.size();
@@ -589,8 +622,8 @@ void BaseSqlTableModel::setTrackValueForColumn(TrackPointer pTrack, int column,
     } else if (fieldIndex(LIBRARYTABLE_BITRATE) == column) {
         pTrack->setBitrate(value.toInt());
     } else if (fieldIndex(LIBRARYTABLE_BPM) == column) {
-        //QVariant::toFloat needs >= QT 4.6.x
-        pTrack->setBpm((float) value.toDouble());
+        // QVariant::toFloat needs >= QT 4.6.x
+        pTrack->setBpm(static_cast<float>(value.toDouble()));
     } else if (fieldIndex(LIBRARYTABLE_PLAYED) == column) {
         pTrack->setPlayed(value.toBool());
     } else if (fieldIndex(LIBRARYTABLE_TIMESPLAYED) == column) {
@@ -603,7 +636,8 @@ void BaseSqlTableModel::setTrackValueForColumn(TrackPointer pTrack, int column,
     }
 }
 
-QVariant BaseSqlTableModel::getBaseValue(const QModelIndex& index, int role) const {
+QVariant BaseSqlTableModel::getBaseValue(
+    const QModelIndex& index, int role) const {
     if (role != Qt::DisplayRole &&
         role != Qt::ToolTipRole &&
         role != Qt::EditRole) {
@@ -653,3 +687,27 @@ QVariant BaseSqlTableModel::getBaseValue(const QModelIndex& index, int role) con
     return QVariant();
 }
 
+QMimeData* BaseSqlTableModel::mimeData(const QModelIndexList &indexes) const {
+    QMimeData *mimeData = new QMimeData();
+    QList<QUrl> urls;
+
+    // The list of indexes we're given contains separates indexes for each
+    // column, so even if only one row is selected, we'll have columnCount()
+    // indices.  We need to only count each row once:
+    QSet<int> rows;
+
+    foreach (QModelIndex index, indexes) {
+        if (!index.isValid() || rows.contains(index.row())) {
+            continue;
+        }
+        rows.insert(index.row());
+        QUrl url = QUrl::fromLocalFile(getTrackLocation(index));
+        if (!url.isValid()) {
+            qDebug() << this << "ERROR: invalid url" << url;
+            continue;
+        }
+        urls.append(url);
+    }
+    mimeData->setUrls(urls);
+    return mimeData;
+}
