@@ -8,28 +8,23 @@
 #include <QSqlTableModel>
 
 #include "library/basesqltablemodel.h"
-#include "library/trackmodel.h"
 
 class TrackCollection;
 
-class CrateTableModel : public BaseSqlTableModel, public virtual TrackModel {
+class CrateTableModel : public BaseSqlTableModel {
     Q_OBJECT
   public:
     CrateTableModel(QObject* parent, TrackCollection* pTrackCollection);
     virtual ~CrateTableModel();
 
     void setCrate(int crateId);
-
-    QMimeData* mimeData(const QModelIndexList &indexes) const;
+    int getCrate() const {
+        return m_iCrateId;
+    }
 
     // From TrackModel
     virtual TrackPointer getTrack(const QModelIndex& index) const;
-    virtual QString getTrackLocation(const QModelIndex& index) const;
-    virtual int getTrackId(const QModelIndex& index) const;
-    virtual const QLinkedList<int> getTrackRows(int trackId) const;
-
     virtual void search(const QString& searchText);
-    virtual const QString currentSearch();
     virtual bool isColumnInternal(int column);
     virtual bool isColumnHiddenByDefault(int column);
     virtual void removeTrack(const QModelIndex& index);
@@ -42,6 +37,7 @@ class CrateTableModel : public BaseSqlTableModel, public virtual TrackModel {
 
   private slots:
     void slotSearch(const QString& searchText);
+
   signals:
     void doSearch(const QString& searchText);
 
@@ -50,5 +46,4 @@ class CrateTableModel : public BaseSqlTableModel, public virtual TrackModel {
     int m_iCrateId;
 };
 
-
 #endif /* CRATETABLEMODEL_H */
