@@ -4,10 +4,11 @@
 #include <QtSql>
 #include "trackinfoobject.h"
 #include "library/dao/playlistdao.h"
+#include "library/queryutil.h"
+#include "library/trackcollection.h"
 
 PlaylistDAO::PlaylistDAO(QSqlDatabase& database)
         : m_database(database) {
-
 }
 
 PlaylistDAO::~PlaylistDAO()
@@ -35,11 +36,10 @@ bool PlaylistDAO::createPlaylist(QString name, bool hidden)
     query.prepare("SELECT max(position) as posmax FROM Playlists");
 
     if (!query.exec()) {
-        qDebug() << query.lastError();
+        LOG_FAILED_QUERY(query);
         m_database.rollback();
         return false;
     }
-    //query.finish();
 
     //Get the id of the last playlist.
     int position = 0;
@@ -48,21 +48,19 @@ bool PlaylistDAO::createPlaylist(QString name, bool hidden)
         position++; //Append after the last playlist.
     }
 
-    qDebug() << "Inserting playlist" << name << "at position" << position;
+    //qDebug() << "Inserting playlist" << name << "at position" << position;
 
     query.prepare("INSERT INTO Playlists (name, position, hidden) "
                   "VALUES (:name, :position, :hidden)");
- 				 // ":date_created, :date_modified)");
     query.bindValue(":name", name);
     query.bindValue(":position", position);
     query.bindValue(":hidden", hidden ? 1 : 0);
 
     if (!query.exec()) {
-        qDebug() << query.lastError();
+        LOG_FAILED_QUERY(query);
         m_database.rollback();
         return false;
     }
-    //query.finish();
 
     int playlistId = query.lastInsertId().toInt();
     //Commit the transaction
@@ -82,7 +80,7 @@ QString PlaylistDAO::getPlaylistName(unsigned int position)
     query.bindValue(":position", position);
 
     if (!query.exec()) {
-        qDebug() << "getPlaylistName" << query.lastError();
+        LOG_FAILED_QUERY(query);
         return "";
     }
 
@@ -91,7 +89,6 @@ QString PlaylistDAO::getPlaylistName(unsigned int position)
     while (query.next()) {
         name = query.value(query.record().indexOf("name")).toString();
     }
-    //query.finish();
     return name;
 }
 
@@ -106,7 +103,7 @@ int PlaylistDAO::getPlaylistIdFromName(QString name) {
             return query.value(query.record().indexOf("id")).toInt();
         }
     } else {
-        qDebug() << "getPlaylistIdFromName:" << query.lastError();
+        LOG_FAILED_QUERY(query);
     }
     return -1;
 }
@@ -126,22 +123,20 @@ void PlaylistDAO::deletePlaylist(int playlistId)
                   "WHERE id= :id");
     query.bindValue(":id", playlistId);
     if (!query.exec()) {
-     	qDebug() << "deletePlaylist" << query.lastError();
-      m_database.rollback();
-     	return;
+        LOG_FAILED_QUERY(query);
+        m_database.rollback();
+        return;
     }
-    //query.finish();
 
     //Delete the tracks in this playlist from the PlaylistTracks table.
     query.prepare("DELETE FROM PlaylistTracks "
                   "WHERE playlist_id = :id");
     query.bindValue(":id", playlistId);
     if (!query.exec()) {
-     	qDebug() << "deletePlaylist" << query.lastError();
-      m_database.rollback();
-     	return;
+        LOG_FAILED_QUERY(query);
+        m_database.rollback();
+        return;
     }
-    //query.finish();
 
     m_database.commit();
     //TODO: Crap, we need to shuffle the positions of all the playlists?
@@ -149,13 +144,57 @@ void PlaylistDAO::deletePlaylist(int playlistId)
     emit(deleted(playlistId));
 }
 
+
+void PlaylistDAO::renamePlaylist(int playlistId, const QString& newName) {
+    QSqlQuery query(m_database);
+    query.prepare("UPDATE Playlists SET name = :name WHERE id = :id");
+    query.bindValue(":name", newName);
+    query.bindValue(":id", playlistId);
+    if (!query.exec()) {
+        LOG_FAILED_QUERY(query);
+    }
+}
+
+
+bool PlaylistDAO::setPlaylistLocked(int playlistId, bool locked) {
+    // SQLite3 doesn't support boolean value. Using integer instead.
+    int lock = locked ? 1 : 0;
+
+    QSqlQuery query(m_database);
+    query.prepare("UPDATE Playlists SET locked = :lock WHERE id = :id");
+    query.bindValue(":lock", lock);
+    query.bindValue(":id", playlistId);
+
+    if (!query.exec()) {
+        LOG_FAILED_QUERY(query);
+        return false;
+    }
+    return true;
+}
+
+bool PlaylistDAO::isPlaylistLocked(int playlistId) {
+    QSqlQuery query(m_database);
+    query.prepare("SELECT locked FROM Playlists WHERE id = :id");
+    query.bindValue(":id", playlistId);
+
+    if (query.exec()) {
+        if (query.next()) {
+            int lockValue = query.value(0).toInt();
+            return lockValue == 1;
+        }
+    } else {
+        LOG_FAILED_QUERY(query);
+    }
+    return false;
+}
+
 /** Append a track to a playlist */
 void PlaylistDAO::appendTrackToPlaylist(int trackId, int playlistId)
 {
     // qDebug() << "PlaylistDAO::appendTrackToPlaylist"
     //          << QThread::currentThread() << m_database.connectionName();
 
-    //Start the transaction
+    // Start the transaction
     m_database.transaction();
 
     //Find out the highest position existing in the playlist so we know what
@@ -164,13 +203,8 @@ void PlaylistDAO::appendTrackToPlaylist(int trackId, int playlistId)
     query.prepare("SELECT max(position) as position FROM PlaylistTracks "
                   "WHERE playlist_id = :id");
     query.bindValue(":id", playlistId);
-    query.exec();
-
-    //Print out any SQL error, if there was one.
-    if (query.lastError().isValid()) {
-     	qDebug() << "appendTrackToPlaylist" << query.lastError();
-      // m_database.rollback();
-      // return;
+    if (!query.exec()) {
+        LOG_FAILED_QUERY(query);
     }
 
     // Get the position of the highest playlist...
@@ -180,16 +214,18 @@ void PlaylistDAO::appendTrackToPlaylist(int trackId, int playlistId)
     }
     position++; //Append after the last song.
 
-
     //Insert the song into the PlaylistTracks table
     query.prepare("INSERT INTO PlaylistTracks (playlist_id, track_id, position)"
                   "VALUES (:playlist_id, :track_id, :position)");
     query.bindValue(":playlist_id", playlistId);
     query.bindValue(":track_id", trackId);
     query.bindValue(":position", position);
-    query.exec();
 
-    //Start the transaction
+    if (!query.exec()) {
+        LOG_FAILED_QUERY(query);
+    }
+
+    // Commit the transaction
     m_database.commit();
 
     emit(trackAdded(playlistId, trackId, position));
@@ -202,9 +238,8 @@ unsigned int PlaylistDAO::playlistCount()
     // qDebug() << "PlaylistDAO::playlistCount" << QThread::currentThread() << m_database.connectionName();
     QSqlQuery query(m_database);
     query.prepare("SELECT count(*) as count FROM Playlists");
-
     if (!query.exec()) {
-        qDebug() << "playlistCount" << query.lastError();
+        LOG_FAILED_QUERY(query);
     }
 
     int numRecords = 0;
@@ -231,7 +266,7 @@ int PlaylistDAO::getPlaylistId(int position)
             }
         }
     } else {
-        qDebug() << query.lastError();
+        LOG_FAILED_QUERY(query);
     }
 
     return -1;
@@ -250,7 +285,7 @@ bool PlaylistDAO::isHidden(int playlistId) {
             return query.value(0).toBool();
         }
     } else {
-        qDebug() << query.lastError();
+        LOG_FAILED_QUERY(query);
     }
     return false;
 }
@@ -263,9 +298,7 @@ void PlaylistDAO::removeTrackFromPlaylists(int trackId) {
             .arg(PLAYLIST_TRACKS_TABLE);
     query.prepare(queryString);
     if (!query.exec()) {
-        qDebug() << __FILE__ << __LINE__
-                 << "Failed to get track playlist membership while removing"
-                 << "track from all playlists:" << trackId;
+        LOG_FAILED_QUERY(query);
         return;
     }
 
@@ -292,7 +325,7 @@ void PlaylistDAO::removeTrackFromPlaylist(int playlistId, int position)
     query.bindValue(":position", position);
 
     if (!query.exec()) {
-        qDebug() << "removeTrackFromPlaylist" << query.lastError();
+        LOG_FAILED_QUERY(query);
         m_database.rollback();
         return;
     }
@@ -311,18 +344,18 @@ void PlaylistDAO::removeTrackFromPlaylist(int playlistId, int position)
     query.bindValue(":position", position);
 
     if (!query.exec()) {
-        qDebug() << "removeTrackFromPlaylist" << query.lastError();
+        LOG_FAILED_QUERY(query);
         m_database.rollback();
         return;
     }
-    //query.finish();
 
     QString queryString;
     queryString = QString("UPDATE PlaylistTracks SET position=position-1 "
                           "WHERE position>=%1 AND "
                           "playlist_id=%2").arg(position).arg(playlistId);
-    query.exec(queryString);
-    //query.finish();
+    if (!query.exec(queryString)) {
+        LOG_FAILED_QUERY(query);
+    }
     m_database.commit();
 
     emit(trackRemoved(playlistId, trackId, position));
@@ -344,7 +377,7 @@ void PlaylistDAO::insertTrackIntoPlaylist(int trackId, int playlistId, int posit
 
     QSqlQuery query(m_database);
     if (!query.exec(queryString)) {
-        qDebug() << "insertTrackIntoPlaylist" << query.lastError();
+        LOG_FAILED_QUERY(query);
     }
 
     //Insert the song into the PlaylistTracks table
@@ -355,13 +388,30 @@ void PlaylistDAO::insertTrackIntoPlaylist(int trackId, int playlistId, int posit
     query.bindValue(":position", position);
 
     if (!query.exec()) {
-        qDebug() << query.lastError();
+        LOG_FAILED_QUERY(query);
     }
-
-    //query.finish();
-
     m_database.commit();
 
     emit(trackAdded(playlistId, trackId, position));
     emit(changed(playlistId));
 }
+
+void PlaylistDAO::addToAutoDJQueue(int playlistId) {
+    //qDebug() << "Adding tracks from playlist " << playlistId << " to the Auto-DJ Queue";
+
+    // Query the PlaylistTracks database to locate tracks in the selected playlist
+    QSqlQuery query(m_database);
+    query.prepare("SELECT track_id FROM PlaylistTracks "
+                  "WHERE playlist_id = :plid");
+    query.bindValue(":plid", playlistId);
+    if (!query.exec()) {
+        LOG_FAILED_QUERY(query);
+    }
+
+    // Get the ID of the Auto-DJ playlist
+    int autoDJId = getPlaylistIdFromName(AUTODJ_TABLE);
+    // Loop through the tracks, adding them to the Auto-DJ Queue
+    while(query.next()) {
+        appendTrackToPlaylist(query.value(0).toInt(), autoDJId);
+    }
+}
