@@ -2,10 +2,14 @@
 #include <QtCore>
 #include <QtSql>
 
-#include "trackinfoobject.h"
 #include "library/dao/trackdao.h"
+
 #include "audiotagger.h"
+#include "library/queryutil.h"
 #include "soundsourceproxy.h"
+#include "track/beatfactory.h"
+#include "track/beats.h"
+#include "trackinfoobject.h"
 
 // The number of tracks to cache in memory at once. Once the n+1'th track is
 // created, the TrackDAO's QCache deletes its TrackPointer to the track, which
@@ -23,29 +27,26 @@ TrackDAO::TrackDAO(QSqlDatabase& database,
           m_cueDao(cueDao),
           m_playlistDao(playlistDao),
           m_crateDao(crateDao),
-          m_pConfig(pConfig),
-          m_trackCache(TRACK_CACHE_SIZE) {
+          m_trackCache(TRACK_CACHE_SIZE),
+          m_pConfig(pConfig) {
 }
 
-void TrackDAO::finish()
-{
+void TrackDAO::finish() {
     //clear out played information on exit
     //crash prevention: if mixxx crashes, played information will be maintained
     qDebug() << "Clearing played information for this session";
     QSqlQuery query(m_database);
-    if (!query.exec("UPDATE library SET played=0"))
-    {
-        qDebug() << "Error clearing played value";
+    if (!query.exec("UPDATE library SET played=0")) {
+        LOG_FAILED_QUERY(query)
+                << "Error clearing played value";
     }
 }
 
-TrackDAO::~TrackDAO()
-{
+TrackDAO::~TrackDAO() {
     qDebug() << "~TrackDAO()";
 }
 
-void TrackDAO::initialize()
-{
+void TrackDAO::initialize() {
     qDebug() << "TrackDAO::initialize" << QThread::currentThread() << m_database.connectionName();
 }
 
@@ -53,8 +54,7 @@ void TrackDAO::initialize()
     @return the track id for the track located at location, or -1 if the track
             is not in the database.
 */
-int TrackDAO::getTrackId(QString absoluteFilePath)
-{
+int TrackDAO::getTrackId(QString absoluteFilePath) {
     //qDebug() << "TrackDAO::getTrackId" << QThread::currentThread() << m_database.connectionName();
 
     QSqlQuery query(m_database);
@@ -62,7 +62,7 @@ int TrackDAO::getTrackId(QString absoluteFilePath)
     query.bindValue(":location", absoluteFilePath);
 
     if (!query.exec()) {
-        qDebug() << query.lastError();
+        LOG_FAILED_QUERY(query);
         return -1;
     }
 
@@ -70,15 +70,13 @@ int TrackDAO::getTrackId(QString absoluteFilePath)
     if (query.next()) {
         libraryTrackId = query.value(query.record().indexOf("id")).toInt();
     }
-    //query.finish();
 
     return libraryTrackId;
 }
 
 /** Some code (eg. drag and drop) needs to just get a track's location, and it's
     not worth retrieving a whole TrackInfoObject.*/
-QString TrackDAO::getTrackLocation(int trackId)
-{
+QString TrackDAO::getTrackLocation(int trackId) {
     qDebug() << "TrackDAO::getTrackLocation"
              << QThread::currentThread() << m_database.connectionName();
     QSqlQuery query(m_database);
@@ -86,13 +84,12 @@ QString TrackDAO::getTrackLocation(int trackId)
     query.prepare("SELECT track_locations.location FROM track_locations INNER JOIN library ON library.location = track_locations.id WHERE library.id=:id");
     query.bindValue(":id", trackId);
     if (!query.exec()) {
-        qDebug() << query.lastError();
+        LOG_FAILED_QUERY(query);
         return "";
     }
     while (query.next()) {
         trackLocation = query.value(query.record().indexOf("location")).toString();
     }
-    //query.finish();
 
     return trackLocation;
 }
@@ -101,15 +98,14 @@ QString TrackDAO::getTrackLocation(int trackId)
     @param file_location The full path to the track on disk, including the filename.
     @return true if the track is found in the library table, false otherwise.
 */
-bool TrackDAO::trackExistsInDatabase(QString absoluteFilePath)
-{
+bool TrackDAO::trackExistsInDatabase(QString absoluteFilePath) {
     return (getTrackId(absoluteFilePath) != -1);
 }
 
 void TrackDAO::saveTrack(TrackPointer track) {
-
-    if (track)
+    if (track) {
         saveTrack(track.data());
+    }
 }
 
 void TrackDAO::saveTrack(TrackInfoObject* pTrack) {
@@ -248,16 +244,17 @@ void TrackDAO::prepareLibraryInsert(QSqlQuery& query) {
                   "filetype, location, comment, url, duration, rating, key, "
                   "bitrate, samplerate, cuepoint, bpm, replaygain, wavesummaryhex, "
                   "timesplayed, "
-                  "channels, mixxx_deleted, header_parsed) "
+                  "channels, mixxx_deleted, header_parsed, beats_version, beats) "
                   "VALUES (:artist, "
                   ":title, :album, :year, :genre, :tracknumber, "
-                  ":filetype, :location, :comment, :url, :duration, :rating, :key,"
+                  ":filetype, :location, :comment, :url, :duration, :rating, :key, "
                   ":bitrate, :samplerate, :cuepoint, :bpm, :replaygain, :wavesummaryhex, "
                   ":timesplayed, "
-                  ":channels, :mixxx_deleted, :header_parsed)");
+                  ":channels, :mixxx_deleted, :header_parsed, :beats_version, :beats)");
 }
 
-void TrackDAO::bindTrackToLibraryInsert(QSqlQuery& query, TrackInfoObject* pTrack, int trackLocationId) {
+void TrackDAO::bindTrackToLibraryInsert(
+    QSqlQuery& query, TrackInfoObject* pTrack, int trackLocationId) {
     query.bindValue(":artist", pTrack->getArtist());
     query.bindValue(":title", pTrack->getTitle());
     query.bindValue(":album", pTrack->getAlbum());
@@ -273,17 +270,33 @@ void TrackDAO::bindTrackToLibraryInsert(QSqlQuery& query, TrackInfoObject* pTrac
     query.bindValue(":bitrate", pTrack->getBitrate());
     query.bindValue(":samplerate", pTrack->getSampleRate());
     query.bindValue(":cuepoint", pTrack->getCuePoint());
-    query.bindValue(":bpm", pTrack->getBpm());
+
     query.bindValue(":replaygain", pTrack->getReplayGain());
     query.bindValue(":key", pTrack->getKey());
     const QByteArray* pWaveSummary = pTrack->getWaveSummary();
-    if (pWaveSummary) //Avoid null pointer deref
-        query.bindValue(":wavesummaryhex", *pWaveSummary);
+    query.bindValue(":wavesummaryhex", pWaveSummary ? *pWaveSummary : QVariant(QVariant::ByteArray));
     query.bindValue(":timesplayed", pTrack->getTimesPlayed());
     //query.bindValue(":datetime_added", pTrack->getDateAdded());
     query.bindValue(":channels", pTrack->getChannels());
     query.bindValue(":mixxx_deleted", 0);
     query.bindValue(":header_parsed", pTrack->getHeaderParsed() ? 1 : 0);
+
+    const QByteArray* pBeatsBlob = NULL;
+    QString blobVersion = "";
+    BeatsPointer pBeats = pTrack->getBeats();
+    // Fall back on cached BPM
+    double dBpm = pTrack->getBpm();
+
+    if (pBeats) {
+        pBeatsBlob = pBeats->toByteArray();
+        blobVersion = pBeats->getVersion();
+        dBpm = pBeats->getBpm();
+    }
+
+    query.bindValue(":bpm", dBpm);
+    query.bindValue(":beats_version", blobVersion);
+    query.bindValue(":beats", pBeatsBlob ? *pBeatsBlob : QVariant(QVariant::ByteArray));
+    delete pBeatsBlob;
 }
 
 void TrackDAO::addTracks(QList<TrackInfoObject*> tracksToAdd, bool unremove) {
@@ -317,7 +330,8 @@ void TrackDAO::addTracks(QList<TrackInfoObject*> tracksToAdd, bool unremove) {
             if (!query_finder.exec()) {
                 // We can't even select this, something is wrong. Skip this
                 // track -- maybe we'll have luck with others.
-                qDebug() << query_finder.lastError();
+                LOG_FAILED_QUERY(query_finder)
+                        << "Can't find track location ID after failing to insert. Something is wrong.";
                 continue;
             }
             while (query_finder.next()) {
@@ -343,9 +357,8 @@ void TrackDAO::addTracks(QList<TrackInfoObject*> tracksToAdd, bool unremove) {
     track_lookup.prepare("SELECT location, id, mixxx_deleted from library WHERE location IN (" +
                          trackLocationIds.join(",")+ ")");
     if (!track_lookup.exec()) {
-        qDebug() << __FILE__ << __LINE__ << "Failed to lookup existing tracks:"
-                 << track_lookup.lastError()
-                 << track_lookup.executedQuery();
+        LOG_FAILED_QUERY(track_lookup)
+                << "Failed to lookup existing tracks:";
     } else {
         QSqlRecord track_lookup_record = track_lookup.record();
         int locationIdColumn = track_lookup_record.indexOf("location");
@@ -384,9 +397,8 @@ void TrackDAO::addTracks(QList<TrackInfoObject*> tracksToAdd, bool unremove) {
                 unremove_query.prepare("UPDATE library SET mixxx_deleted = 0 WHERE id = :id");
                 unremove_query.bindValue(":id", trackId);
                 if (!unremove_query.exec()) {
-                    qDebug() << __FILE__ << __LINE__ << "Could not unremove track" << trackId
-                             << unremove_query.lastError()
-                             << unremove_query.executedQuery();
+                    LOG_FAILED_QUERY(unremove_query)
+                            << "Could not unremove track" << trackId;
                 } else {
                     tracksAddedSet.insert(trackId);
                 }
@@ -407,8 +419,9 @@ void TrackDAO::addTracks(QList<TrackInfoObject*> tracksToAdd, bool unremove) {
         if (!query.exec()) {
             // We failed to insert the track. Maybe it is already in the library
             // but marked deleted? Skip this track.
-            qDebug() << "Failed to INSERT new track into library"
-                     << __FILE__ << __LINE__ << query.lastError();
+            LOG_FAILED_QUERY(query)
+                    << "Failed to INSERT new track into library:"
+                    << pTrack->getFilename();
             continue;
         }
         int trackId = query.lastInsertId().toInt();
@@ -431,7 +444,7 @@ int TrackDAO::addTrack(QFileInfo& fileInfo, bool unremove) {
     int trackId = -1;
     TrackInfoObject * pTrack = new TrackInfoObject(fileInfo);
     if (pTrack) {
-        //Add the song to the database.
+        // Add the song to the database.
         addTrack(pTrack, unremove);
         trackId = pTrack->getId();
         delete pTrack;
@@ -451,9 +464,8 @@ void TrackDAO::addTrack(TrackInfoObject* pTrack, bool unremove) {
     addTracks(tracksToAdd, unremove);
 }
 
-  /** Removes a track from the library track collection. */
-void TrackDAO::removeTrack(int id)
-{
+/** Removes a track from the library track collection. */
+void TrackDAO::removeTrack(int id) {
     //qDebug() << "TrackDAO::removeTrack" << QThread::currentThread() << m_database.connectionName();
     Q_ASSERT(id >= 0);
     QSqlQuery query(m_database);
@@ -466,12 +478,8 @@ void TrackDAO::removeTrack(int id)
     query.prepare("UPDATE library "
                   "SET mixxx_deleted=1 "
                   "WHERE id = " + QString::number(id));
-    query.exec();
-    //query.finish();
-
-    //Print out any SQL error, if there was one.
-    if (query.lastError().isValid()) {
-        qDebug() << query.lastError();
+    if (!query.exec()) {
+        LOG_FAILED_QUERY(query);
     }
 
     QSet<int> tracksRemovedSet;
@@ -494,7 +502,7 @@ void TrackDAO::removeTracks(QList<int> ids) {
     QSqlQuery query(m_database);
     query.prepare(QString("UPDATE library SET mixxx_deleted=1 WHERE id in (%1)").arg(idList));
     if (!query.exec()) {
-        qDebug() << query.lastError();
+        LOG_FAILED_QUERY(query);
     }
 
     QSet<int> tracksRemovedSet = QSet<int>::fromList(ids);
@@ -508,15 +516,15 @@ void TrackDAO::removeTracks(QList<int> ids) {
      This function should get called if you drag-and-drop a file that's been
      "removed" from Mixxx back into the library view.
 */
-void TrackDAO::unremoveTrack(int trackId)
-{
+void TrackDAO::unremoveTrack(int trackId) {
     Q_ASSERT(trackId >= 0);
     QSqlQuery query(m_database);
     query.prepare("UPDATE library "
                   "SET mixxx_deleted=0 "
                   "WHERE id = " + QString("%1").arg(trackId));
     if (!query.exec()) {
-        qDebug() << "Failed to set track" << trackId << "as undeleted" << query.lastError();
+        LOG_FAILED_QUERY(query)
+                << "Failed to set track" << trackId << "as undeleted";
     }
     QSet<int> tracksAddedSet;
     tracksAddedSet.insert(trackId);
@@ -526,7 +534,6 @@ void TrackDAO::unremoveTrack(int trackId)
 // static
 void TrackDAO::deleteTrack(TrackInfoObject* pTrack) {
     Q_ASSERT(pTrack);
-
     //qDebug() << "Garbage Collecting" << pTrack << "ID" << pTrack->getId() << pTrack->getInfo();
 
     // Save the track if it is dirty.
@@ -537,19 +544,14 @@ void TrackDAO::deleteTrack(TrackInfoObject* pTrack) {
 }
 
 TrackPointer TrackDAO::getTrackFromDB(QSqlQuery &query) const {
-    if (!query.isValid()) {
-        //query.exec();
-    }
-
     //Print out any SQL error, if there was one.
     if (query.lastError().isValid()) {
-        qDebug() << query.lastError();
+        LOG_FAILED_QUERY(query);
     }
 
-    int locationId = -1;
     while (query.next()) {
+        // Good god! Assign query.record() to a freaking variable!
         int trackId = query.value(query.record().indexOf("id")).toInt();
-
         QString artist = query.value(query.record().indexOf("artist")).toString();
         QString title = query.value(query.record().indexOf("title")).toString();
         QString album = query.value(query.record().indexOf("album")).toString();
@@ -571,65 +573,71 @@ TrackPointer TrackDAO::getTrackFromDB(QSqlQuery &query) const {
         int timesplayed = query.value(query.record().indexOf("timesplayed")).toInt();
         int played = query.value(query.record().indexOf("played")).toInt();
         int channels = query.value(query.record().indexOf("channels")).toInt();
-        int filesize = query.value(query.record().indexOf("filesize")).toInt();
+        //int filesize = query.value(query.record().indexOf("filesize")).toInt();
         QString filetype = query.value(query.record().indexOf("filetype")).toString();
         QString location = query.value(query.record().indexOf("location")).toString();
         bool header_parsed = query.value(query.record().indexOf("header_parsed")).toBool();
         QDateTime date_created = query.value(query.record().indexOf("datetime_added")).toDateTime();
 
-        TrackInfoObject* track = new TrackInfoObject(location, false);
+       TrackPointer pTrack = TrackPointer(new TrackInfoObject(location, false), &TrackDAO::deleteTrack);
 
         // TIO already stats the file to see if it exists, what its length is,
         // etc. So don't bother setting it.
         //track->setLength(filesize);
 
-        track->setId(trackId);
-        track->setArtist(artist);
-        track->setTitle(title);
-        track->setAlbum(album);
-        track->setYear(year);
-        track->setGenre(genre);
-        track->setTrackNumber(tracknumber);
-        track->setRating(rating);
-        track->setKey(key);
+        pTrack->setId(trackId);
+        pTrack->setArtist(artist);
+        pTrack->setTitle(title);
+        pTrack->setAlbum(album);
+        pTrack->setYear(year);
+        pTrack->setGenre(genre);
+        pTrack->setTrackNumber(tracknumber);
+        pTrack->setRating(rating);
+        pTrack->setKey(key);
 
-        track->setComment(comment);
-        track->setURL(url);
-        track->setDuration(duration);
-        track->setBitrate(bitrate);
-        track->setSampleRate(samplerate);
-        track->setCuePoint((float)cuepoint);
-        track->setBpm(bpm.toFloat());
-        track->setReplayGain(replaygain.toFloat());
-        track->setWaveSummary(wavesummaryhex, false);
+        pTrack->setComment(comment);
+        pTrack->setURL(url);
+        pTrack->setDuration(duration);
+        pTrack->setBitrate(bitrate);
+        pTrack->setSampleRate(samplerate);
+        pTrack->setCuePoint((float)cuepoint);
+        pTrack->setBpm(bpm.toFloat());
+        pTrack->setReplayGain(replaygain.toFloat());
+        pTrack->setWaveSummary(wavesummaryhex, false);
         delete wavesummaryhex;
-        track->setTimesPlayed(timesplayed);
-        track->setPlayed(played);
-        track->setChannels(channels);
-        track->setType(filetype);
-        track->setLocation(location);
-        track->setHeaderParsed(header_parsed);
-        track->setDateAdded(date_created);
 
-        track->setCuePoints(m_cueDao.getCuesForTrack(trackId));
-        track->setDirty(false);
+        QString beatsVersion = query.value(query.record().indexOf("beats_version")).toString();
+        QByteArray beatsBlob = query.value(query.record().indexOf("beats")).toByteArray();
+        BeatsPointer pBeats = BeatFactory::loadBeatsFromByteArray(pTrack, beatsVersion, &beatsBlob);
+        if (pBeats) {
+            pTrack->setBeats(pBeats);
+        }
+
+        pTrack->setTimesPlayed(timesplayed);
+        pTrack->setPlayed(played);
+        pTrack->setChannels(channels);
+        pTrack->setType(filetype);
+        pTrack->setLocation(location);
+        pTrack->setHeaderParsed(header_parsed);
+        pTrack->setDateAdded(date_created);
+
+        pTrack->setCuePoints(m_cueDao.getCuesForTrack(trackId));
+        pTrack->setDirty(false);
 
         // Listen to dirty and changed signals
-        connect(track, SIGNAL(dirty(TrackInfoObject*)),
+        connect(pTrack.data(), SIGNAL(dirty(TrackInfoObject*)),
                 this, SLOT(slotTrackDirty(TrackInfoObject*)),
                 Qt::DirectConnection);
-        connect(track, SIGNAL(clean(TrackInfoObject*)),
+        connect(pTrack.data(), SIGNAL(clean(TrackInfoObject*)),
                 this, SLOT(slotTrackClean(TrackInfoObject*)),
                 Qt::DirectConnection);
-        connect(track, SIGNAL(changed(TrackInfoObject*)),
+        connect(pTrack.data(), SIGNAL(changed(TrackInfoObject*)),
                 this, SLOT(slotTrackChanged(TrackInfoObject*)),
                 Qt::DirectConnection);
-        connect(track, SIGNAL(save(TrackInfoObject*)),
+        connect(pTrack.data(), SIGNAL(save(TrackInfoObject*)),
                 this, SLOT(slotTrackSave(TrackInfoObject*)),
                 Qt::DirectConnection);
 
-        TrackPointer pTrack = TrackPointer(track, &TrackDAO::deleteTrack);
-
         // Automatic conversion to a weak pointer
         m_tracks[trackId] = pTrack;
         m_trackCache.insert(trackId, new TrackPointer(pTrack));
@@ -641,15 +649,22 @@ TrackPointer TrackDAO::getTrackFromDB(QSqlQuery &query) const {
             pTrack->parse();
         }
 
+        if (!pBeats && pTrack->getBpm() != 0.0f) {
+            // The track has no stored beats but has a previously detected BPM
+            // or a BPM loaded from metadata. Automatically create a beatgrid
+            // for the track. This dirties the track so we have to do it after
+            // all the signal connections, etc. are in place.
+            BeatsPointer pBeats = BeatFactory::makeBeatGrid(pTrack, pTrack->getBpm(), 0.0f);
+            pTrack->setBeats(pBeats);
+        }
+
         return pTrack;
     }
-    //query.finish();
 
     return TrackPointer();
 }
 
-TrackPointer TrackDAO::getTrack(int id, bool cacheOnly) const
-{
+TrackPointer TrackDAO::getTrack(int id, bool cacheOnly) const {
     //qDebug() << "TrackDAO::getTrack" << QThread::currentThread() << m_database.connectionName();
 
     // If the track cache contains the track, use it to get a strong reference
@@ -688,14 +703,25 @@ TrackPointer TrackDAO::getTrack(int id, bool cacheOnly) const
     time.start();
     QSqlQuery query(m_database);
 
-    query.prepare("SELECT library.id, artist, title, album, year, genre, tracknumber, filetype, rating, key, track_locations.location as location, track_locations.filesize as filesize, comment, url, duration, bitrate, samplerate, cuepoint, bpm, replaygain, wavesummaryhex, channels, header_parsed, timesplayed, played, datetime_added FROM Library INNER JOIN track_locations ON library.location = track_locations.id WHERE library.id=" + QString("%1").arg(id));
+    query.prepare(
+        "SELECT library.id, artist, title, album, year, genre, tracknumber, "
+        "filetype, rating, key, track_locations.location as location, "
+        "track_locations.filesize as filesize, comment, url, duration, bitrate, "
+        "samplerate, cuepoint, bpm, replaygain, wavesummaryhex, channels, "
+        "header_parsed, timesplayed, played, beats_version, beats, datetime_added "
+        "FROM Library "
+        "INNER JOIN track_locations "
+            "ON library.location = track_locations.id "
+        "WHERE library.id=" + QString("%1").arg(id)
+    );
 
     TrackPointer pTrack;
 
     if (query.exec()) {
          pTrack = getTrackFromDB(query);
     } else {
-        qDebug() << QString("getTrack(%1)").arg(id) << query.lastError();
+        LOG_FAILED_QUERY(query)
+                << QString("getTrack(%1)").arg(id);
     }
     //qDebug() << "getTrack hit the database, took " << time.elapsed() << "ms";
 
@@ -703,8 +729,7 @@ TrackPointer TrackDAO::getTrack(int id, bool cacheOnly) const
 }
 
 /** Saves a track's info back to the database */
-void TrackDAO::updateTrack(TrackInfoObject* pTrack)
-{
+void TrackDAO::updateTrack(TrackInfoObject* pTrack) {
     m_database.transaction();
     QTime time;
     time.start();
@@ -727,7 +752,8 @@ void TrackDAO::updateTrack(TrackInfoObject* pTrack)
                   "bitrate=:bitrate, samplerate=:samplerate, cuepoint=:cuepoint, "
                   "bpm=:bpm, replaygain=:replaygain, wavesummaryhex=:wavesummaryhex, "
                   "timesplayed=:timesplayed, played=:played, "
-                  "channels=:channels, header_parsed=:header_parsed "
+                  "channels=:channels, header_parsed=:header_parsed, "
+                  "beats_version=:beats_version, beats=:beats "
                   "WHERE id="+QString("%1").arg(trackId));
     query.bindValue(":artist", pTrack->getArtist());
     query.bindValue(":title", pTrack->getTitle());
@@ -742,21 +768,36 @@ void TrackDAO::updateTrack(TrackInfoObject* pTrack)
     query.bindValue(":bitrate", pTrack->getBitrate());
     query.bindValue(":samplerate", pTrack->getSampleRate());
     query.bindValue(":cuepoint", pTrack->getCuePoint());
-    query.bindValue(":bpm", pTrack->getBpm());
+
     query.bindValue(":replaygain", pTrack->getReplayGain());
     query.bindValue(":key", pTrack->getKey());
     query.bindValue(":rating", pTrack->getRating());
     const QByteArray* pWaveSummary = pTrack->getWaveSummary();
-    if (pWaveSummary) //Avoid null pointer deref
-        query.bindValue(":wavesummaryhex", *pWaveSummary);
+    query.bindValue(":wavesummaryhex", pWaveSummary ? *pWaveSummary : QVariant(QVariant::ByteArray));
     query.bindValue(":timesplayed", pTrack->getTimesPlayed());
     query.bindValue(":played", pTrack->getPlayed());
     query.bindValue(":channels", pTrack->getChannels());
     query.bindValue(":header_parsed", pTrack->getHeaderParsed() ? 1 : 0);
     //query.bindValue(":location", pTrack->getLocation());
 
+    BeatsPointer pBeats = pTrack->getBeats();
+    QByteArray* pBeatsBlob = NULL;
+    QString beatsVersion = "";
+    double dBpm = pTrack->getBpm();
+
+    if (pBeats) {
+        pBeatsBlob = pBeats->toByteArray();
+        beatsVersion = pBeats->getVersion();
+        dBpm = pBeats->getBpm();
+    }
+
+    query.bindValue(":beats", pBeatsBlob ? *pBeatsBlob : QVariant(QVariant::ByteArray));
+    query.bindValue(":beats_version", beatsVersion);
+    query.bindValue(":bpm", dBpm);
+    delete pBeatsBlob;
+
     if (!query.exec()) {
-        qDebug() << query.lastError();
+        LOG_FAILED_QUERY(query);
         m_database.rollback();
         return;
     }
@@ -767,8 +808,6 @@ void TrackDAO::updateTrack(TrackInfoObject* pTrack)
         return;
     }
 
-    //query.finish();
-
     //qDebug() << "Update track took : " << time.elapsed() << "ms. Now updating cues";
     time.start();
     m_cueDao.saveTrackCues(trackId, pTrack);
@@ -783,8 +822,7 @@ void TrackDAO::updateTrack(TrackInfoObject* pTrack)
 /** Mark all the tracks whose paths begin with libraryPath as invalid.
     That means we'll need to later check that those tracks actually
     (still) exist as part of the library scanning procedure. */
-void TrackDAO::invalidateTrackLocationsInLibrary(QString libraryPath)
-{
+void TrackDAO::invalidateTrackLocationsInLibrary(QString libraryPath) {
     //qDebug() << "TrackDAO::invalidateTrackLocations" << QThread::currentThread() << m_database.connectionName();
     //qDebug() << "invalidateTrackLocations(" << libraryPath << ")";
     libraryPath += "%"; //Add wildcard to SQL query to match subdirectories!
@@ -795,7 +833,9 @@ void TrackDAO::invalidateTrackLocationsInLibrary(QString libraryPath)
                   "WHERE directory LIKE :directory");
     query.bindValue(":directory", libraryPath);
     if (!query.exec()) {
-        qDebug() << "Couldn't mark tracks in directory" << libraryPath <<  "as needing verification." << query.lastError();
+        LOG_FAILED_QUERY(query)
+                << "Couldn't mark tracks in directory" << libraryPath
+                <<  "as needing verification.";
     }
 }
 
@@ -810,12 +850,12 @@ void TrackDAO::markTrackLocationAsVerified(QString location)
                   "WHERE location=:location");
     query.bindValue(":location", location);
     if (!query.exec()) {
-        qDebug() << "Couldn't mark track" << location << " as verified." << query.lastError();
+        LOG_FAILED_QUERY(query)
+                << "Couldn't mark track" << location << " as verified.";
     }
 }
 
-void TrackDAO::markTracksInDirectoryAsVerified(QString directory)
-{
+void TrackDAO::markTracksInDirectoryAsVerified(QString directory) {
     //qDebug() << "TrackDAO::markTracksInDirectoryAsVerified" << QThread::currentThread() << m_database.connectionName();
     //qDebug() << "markTracksInDirectoryAsVerified()" << directory;
 
@@ -825,12 +865,12 @@ void TrackDAO::markTracksInDirectoryAsVerified(QString directory)
                   "WHERE directory=:directory");
     query.bindValue(":directory", directory);
     if (!query.exec()) {
-        qDebug() << "Couldn't mark tracks in" << directory << " as verified." << query.lastError();
+        LOG_FAILED_QUERY(query)
+                << "Couldn't mark tracks in" << directory << " as verified.";
     }
 }
 
-void TrackDAO::markUnverifiedTracksAsDeleted()
-{
+void TrackDAO::markUnverifiedTracksAsDeleted() {
     //qDebug() << "TrackDAO::markUnverifiedTracksAsDeleted" << QThread::currentThread() << m_database.connectionName();
     //qDebug() << "markUnverifiedTracksAsDeleted()";
 
@@ -839,13 +879,13 @@ void TrackDAO::markUnverifiedTracksAsDeleted()
                   "SET fs_deleted=1, needs_verification=0 "
                   "WHERE needs_verification=1");
     if (!query.exec()) {
-        qDebug() << "Couldn't mark unverified tracks as deleted." << query.lastError();
+        LOG_FAILED_QUERY(query)
+                << "Couldn't mark unverified tracks as deleted.";
     }
 
 }
 
-void TrackDAO::markTrackLocationsAsDeleted(QString directory)
-{
+void TrackDAO::markTrackLocationsAsDeleted(QString directory) {
     //qDebug() << "TrackDAO::markTrackLocationsAsDeleted" << QThread::currentThread() << m_database.connectionName();
     QSqlQuery query(m_database);
     query.prepare("UPDATE track_locations "
@@ -853,7 +893,8 @@ void TrackDAO::markTrackLocationsAsDeleted(QString directory)
                   "WHERE directory=:directory");
     query.bindValue(":directory", directory);
     if (!query.exec()) {
-        qDebug() << "Couldn't mark tracks in" << directory << "as deleted." << query.lastError();
+        LOG_FAILED_QUERY(query)
+                << "Couldn't mark tracks in" << directory << "as deleted.";
     }
 }
 
@@ -861,8 +902,7 @@ void TrackDAO::markTrackLocationsAsDeleted(QString directory)
     and see if another "file" with the same name and filesize exists in the track_locations
     table. That means the file has moved instead of being deleted outright, and so
     we can salvage your existing metadata that you have in your DB (like cue points, etc.). */
-void TrackDAO::detectMovedFiles()
-{
+void TrackDAO::detectMovedFiles() {
     //This function should not start a transaction on it's own!
     //When it's called from libraryscanner.cpp, there already is a transaction
     //started!
@@ -875,7 +915,10 @@ void TrackDAO::detectMovedFiles()
     int fileSize;
 
     query.prepare("SELECT id, filename, filesize FROM track_locations WHERE fs_deleted=1");
-    query.exec();
+
+    if (!query.exec()) {
+        LOG_FAILED_QUERY(query);
+    }
 
     //For each track that's been "deleted" on disk...
     while (query.next()) {
@@ -929,8 +972,7 @@ void TrackDAO::detectMovedFiles()
     }
 }
 
-void TrackDAO::clearCache()
-{
+void TrackDAO::clearCache() {
     m_trackCache.clear();
     m_dirtyTracks.clear();
 }
