@@ -27,6 +27,7 @@
 
 LibraryScanner::LibraryScanner(TrackCollection* collection) :
     m_pCollection(collection),
+    m_pProgress(NULL),
     m_libraryHashDao(m_database),
     m_cueDao(m_database),
     m_playlistDao(m_database),
@@ -139,8 +140,12 @@ void LibraryScanner::run()
     //Lower our priority to help not grind crappy computers.
     setPriority(QThread::LowPriority);
 
+
+    if (!m_database.isValid()) {
+       m_database = QSqlDatabase::addDatabase("QSQLITE", "LIBRARY_SCANNER");
+    }
+
     if (!m_database.isOpen()) {
-        m_database = QSqlDatabase::addDatabase("QSQLITE", "LIBRARY_SCANNER");
         m_database.setHostName("localhost");
         m_database.setDatabaseName(MIXXX_DB_PATH);
         m_database.setUserName("mixxx");
@@ -297,6 +302,7 @@ void LibraryScanner::scan(QString libraryPath)
 {
     m_qLibraryPath = libraryPath;
     m_pProgress = new LibraryScannerDlg();
+    m_pProgress->setAttribute(Qt::WA_DeleteOnClose);
 
     //The important part here is that we need to use
     //Qt::BlockingQueuedConnection, because we're sending these signals across
