@@ -1,17 +1,19 @@
 #include <QtCore>
 #include <QtGui>
 #include <QtSql>
+#include <QDateTime>
 #include "library/trackcollection.h"
 #include "library/playlisttablemodel.h"
+#include "library/queryutil.h"
 
 #include "mixxxutils.cpp"
 
 PlaylistTableModel::PlaylistTableModel(QObject* parent,
-                                       TrackCollection* pTrackCollection)
-        : TrackModel(pTrackCollection->getDatabase(),
-                     "mixxx.db.model.playlist"),
-          BaseSqlTableModel(parent, pTrackCollection,
-                            pTrackCollection->getDatabase()),
+                                       TrackCollection* pTrackCollection,
+                                       QString settingsNamespace)
+        : BaseSqlTableModel(parent, pTrackCollection,
+                            pTrackCollection->getDatabase(),
+                            settingsNamespace),
           m_pTrackCollection(pTrackCollection),
           m_playlistDao(m_pTrackCollection->getPlaylistDAO()),
           m_trackDao(m_pTrackCollection->getTrackDAO()),
@@ -23,7 +25,6 @@ PlaylistTableModel::PlaylistTableModel(QObject* parent,
 PlaylistTableModel::~PlaylistTableModel() {
 }
 
-
 void PlaylistTableModel::setPlaylist(int playlistId) {
     //qDebug() << "PlaylistTableModel::setPlaylist" << playlistId;
 
@@ -33,60 +34,43 @@ void PlaylistTableModel::setPlaylist(int playlistId) {
     }
 
     m_iPlaylistId = playlistId;
-
-    QString playlistTableName = "playlist_" + QString("%1").arg(m_iPlaylistId);
+    QString playlistTableName = "playlist_" + QString::number(m_iPlaylistId);
     QSqlQuery query(m_pTrackCollection->getDatabase());
-    //query.prepare("DROP VIEW " + playlistTableName);
-    //query.exec();
-
-    // Escape the playlist name
-    QSqlDriver* driver = m_pTrackCollection->getDatabase().driver();
-    QSqlField playlistNameField("name", QVariant::String);
-    playlistNameField.setValue(playlistTableName);
+    FieldEscaper escaper(m_pTrackCollection->getDatabase());
 
     QStringList columns;
-    columns << "library." + LIBRARYTABLE_ID
-            << "PlaylistTracks." + PLAYLISTTRACKSTABLE_POSITION;
+    columns << PLAYLISTTRACKSTABLE_TRACKID
+            << PLAYLISTTRACKSTABLE_POSITION;
 
-    query.prepare("CREATE TEMPORARY VIEW IF NOT EXISTS " +
-                  driver->formatValue(playlistNameField) + " AS "
-                  "SELECT "
-                  + columns.join(",") +
-                  " FROM library "
-                  "INNER JOIN PlaylistTracks "
-                  "ON library.id = PlaylistTracks.track_id "
-                  "INNER JOIN track_locations "
-                  "ON library.location = track_locations.id "
-                  "WHERE (PlaylistTracks.playlist_id = " + QString("%1").arg(playlistId) +
-                  ") AND (" +  LibraryTableModel::DEFAULT_LIBRARYFILTER +
-                  ") ORDER BY PlaylistTracks.position ");
-
-    //query.bindValue(":playlist_name", playlistTableName);
-    //query.bindValue(":playlist_id", m_iPlaylistId);
+    // We drop files that have been explicitly deleted from mixxx
+    // (mixxx_deleted=0) from the view. There was a bug in <= 1.9.0 where
+    // removed files were not removed from playlists, so some users will have
+    // libraries where this is the case.
+    QString queryString = QString(
+        "CREATE TEMPORARY VIEW IF NOT EXISTS %1 AS "
+        "SELECT %2 FROM PlaylistTracks "
+        "INNER JOIN library ON library.id = PlaylistTracks.track_id "
+        "WHERE PlaylistTracks.playlist_id = %3 AND library.mixxx_deleted = 0")
+            .arg(escaper.escapeString(playlistTableName))
+            .arg(columns.join(","))
+            .arg(playlistId);
+    query.prepare(queryString);
     if (!query.exec()) {
-        // It's normal for this to fail.
-        qDebug() << query.executedQuery() << query.lastError();
+        LOG_FAILED_QUERY(query);
     }
 
-    // Print out any SQL error, if there was one.
-    if (query.lastError().isValid()) {
-     	qDebug() << __FILE__ << __LINE__ << query.lastError();
-    }
-
-    QStringList tableColumns;
-    tableColumns << LIBRARYTABLE_ID;
-    tableColumns << PLAYLISTTRACKSTABLE_POSITION;
-    setTable(playlistTableName, LIBRARYTABLE_ID, tableColumns,
+    setTable(playlistTableName, columns[0], columns,
              m_pTrackCollection->getTrackSource("default"));
     initHeaderData();
     setSearch("");
+    setDefaultSort(fieldIndex("position"), Qt::AscendingOrder);
 }
 
 bool PlaylistTableModel::addTrack(const QModelIndex& index, QString location) {
-    const int positionColumnIndex = this->fieldIndex(PLAYLISTTRACKSTABLE_POSITION);
-    int position = index.sibling(index.row(), positionColumnIndex).data().toInt();
+    const int positionColumn = fieldIndex(PLAYLISTTRACKSTABLE_POSITION);
+    int position = index.sibling(index.row(), positionColumn).data().toInt();
 
-    //Handle weird cases like a drag-and-drop to an invalid index
+    // Handle weird cases like a drag-and-drop to an invalid index
     if (position <= 0) {
         position = rowCount() + 1;
     }
@@ -107,12 +91,10 @@ bool PlaylistTableModel::addTrack(const QModelIndex& index, QString location) {
 
     // TODO(rryan) signal an add to the base, don't select
     select(); //Repopulate the data model.
-
     return true;
 }
 
-TrackPointer PlaylistTableModel::getTrack(const QModelIndex& index) const
-{
+TrackPointer PlaylistTableModel::getTrack(const QModelIndex& index) const {
     //FIXME: use position instead of location for playlist tracks?
 
     //const int locationColumnIndex = this->fieldIndex(LIBRARYTABLE_LOCATION);
@@ -121,34 +103,24 @@ TrackPointer PlaylistTableModel::getTrack(const QModelIndex& index) const
     return m_trackDao.getTrack(trackId);
 }
 
-QString PlaylistTableModel::getTrackLocation(const QModelIndex& index) const
-{
-    QString location = index.sibling(index.row(), fieldIndex("location")).data().toString();
-    return location;
-}
-
-int PlaylistTableModel::getTrackId(const QModelIndex& index) const
-{
-    if (!index.isValid()) {
-        return -1;
+void PlaylistTableModel::removeTrack(const QModelIndex& index) {
+    if (m_playlistDao.isPlaylistLocked(m_iPlaylistId)) {
+        return;
     }
-    return index.sibling(index.row(), fieldIndex(LIBRARYTABLE_ID)).data().toInt();
-}
 
-const QLinkedList<int> PlaylistTableModel::getTrackRows(int trackId) const {
-    return BaseSqlTableModel::getTrackRows(trackId);
-}
-
-void PlaylistTableModel::removeTrack(const QModelIndex& index)
-{
     const int positionColumnIndex = fieldIndex(PLAYLISTTRACKSTABLE_POSITION);
     int position = index.sibling(index.row(), positionColumnIndex).data().toInt();
     m_playlistDao.removeTrackFromPlaylist(m_iPlaylistId, position);
-
     select(); //Repopulate the data model.
 }
 
 void PlaylistTableModel::removeTracks(const QModelIndexList& indices) {
+    bool locked = m_playlistDao.isPlaylistLocked(m_iPlaylistId);
+
+    if (locked) {
+        return;
+    }
+
     const int positionColumnIndex = fieldIndex(PLAYLISTTRACKSTABLE_POSITION);
 
     QList<int> trackPositions;
@@ -169,8 +141,8 @@ void PlaylistTableModel::removeTracks(const QModelIndexList& indices) {
     select();
 }
 
-void PlaylistTableModel::moveTrack(const QModelIndex& sourceIndex, const QModelIndex& destIndex)
-{
+void PlaylistTableModel::moveTrack(const QModelIndex& sourceIndex,
+                                   const QModelIndex& destIndex) {
     //QSqlRecord sourceRecord = this->record(sourceIndex.row());
     //sourceRecord.setValue("position", destIndex.row());
     //this->removeRows(sourceIndex.row(), 1);
@@ -276,6 +248,36 @@ void PlaylistTableModel::moveTrack(const QModelIndex& sourceIndex, const QModelI
     select();
 }
 
+void PlaylistTableModel::shuffleTracks(const QModelIndex& currentIndex) {
+    int numOfTracks = rowCount();
+    int seed = QDateTime::currentDateTime().toTime_t();
+    qsrand(seed);
+    QSqlQuery query(m_pTrackCollection->getDatabase());
+    const int positionColumnIndex = fieldIndex(PLAYLISTTRACKSTABLE_POSITION);
+    int currentPosition = currentIndex.sibling(currentIndex.row(), positionColumnIndex).data().toInt();
+    int shuffleStartIndex = currentPosition + 1;
+
+    m_pTrackCollection->getDatabase().transaction();
+
+    // This is a simple Fisher-Yates shuffling algorithm
+    for (int i=numOfTracks-1; i >= shuffleStartIndex; i--)
+    {
+        int random = int(qrand() / (RAND_MAX + 1.0) * (numOfTracks + 1 - shuffleStartIndex) + shuffleStartIndex);
+        qDebug() << "Swapping tracks " << i << " and " << random;
+        QString swapQuery = "UPDATE PlaylistTracks SET position=%1 WHERE position=%2 AND playlist_id=%3";
+        query.exec(swapQuery.arg(-1).arg(i).arg(m_iPlaylistId));
+        query.exec(swapQuery.arg(i).arg(random).arg(m_iPlaylistId));
+        query.exec(swapQuery.arg(random).arg(-1).arg(m_iPlaylistId));
+
+        if (query.lastError().isValid())
+            qDebug() << query.lastError();
+    }
+
+    m_pTrackCollection->getDatabase().commit();
+    // TODO(XXX) set dirty because someday select() will only do work on dirty.
+    select();
+}
+
 void PlaylistTableModel::search(const QString& searchText) {
     // qDebug() << "PlaylistTableModel::search()" << searchText
     //          << QThread::currentThread();
@@ -283,15 +285,12 @@ void PlaylistTableModel::search(const QString& searchText) {
 }
 
 void PlaylistTableModel::slotSearch(const QString& searchText) {
-    BaseSqlTableModel::search(searchText);
-}
-
-const QString PlaylistTableModel::currentSearch() {
-    return BaseSqlTableModel::currentSearch();
+    BaseSqlTableModel::search(
+        searchText, LibraryTableModel::DEFAULT_LIBRARYFILTER);
 }
 
 bool PlaylistTableModel::isColumnInternal(int column) {
-    if (column == fieldIndex(LIBRARYTABLE_ID) ||
+    if (column == fieldIndex(PLAYLISTTRACKSTABLE_TRACKID) ||
         column == fieldIndex(LIBRARYTABLE_PLAYED) ||
         column == fieldIndex(LIBRARYTABLE_MIXXXDELETED) ||
         column == fieldIndex(TRACKLOCATIONSTABLE_FSDELETED)) {
@@ -306,43 +305,31 @@ bool PlaylistTableModel::isColumnHiddenByDefault(int column) {
     return false;
 }
 
-QMimeData* PlaylistTableModel::mimeData(const QModelIndexList &indexes) const {
-    QMimeData *mimeData = new QMimeData();
-    QList<QUrl> urls;
-
-    //Ok, so the list of indexes we're given contains separates indexes for
-    //each column, so even if only one row is selected, we'll have like 7 indexes.
-    //We need to only count each row once:
-    QList<int> rows;
-
-    foreach (QModelIndex index, indexes) {
-        if (index.isValid()) {
-            if (!rows.contains(index.row())) {
-                rows.push_back(index.row());
-                QUrl url = QUrl::fromLocalFile(getTrackLocation(index));
-                if (!url.isValid())
-                    qDebug() << "ERROR invalid url\n";
-                else
-                    urls.append(url);
-            }
-        }
-    }
-    mimeData->setUrls(urls);
-    return mimeData;
-}
-
-
 QItemDelegate* PlaylistTableModel::delegateForColumn(const int i) {
     return NULL;
 }
 
-TrackModel::CapabilitiesFlags PlaylistTableModel::getCapabilities() const
-{
-    TrackModel::CapabilitiesFlags caps = TRACKMODELCAPS_RECEIVEDROPS | TRACKMODELCAPS_REORDER | TRACKMODELCAPS_ADDTOCRATE | TRACKMODELCAPS_ADDTOPLAYLIST;
+TrackModel::CapabilitiesFlags PlaylistTableModel::getCapabilities() const {
+    TrackModel::CapabilitiesFlags caps = TRACKMODELCAPS_NONE
+            | TRACKMODELCAPS_RECEIVEDROPS
+            | TRACKMODELCAPS_REORDER
+            | TRACKMODELCAPS_ADDTOCRATE
+            | TRACKMODELCAPS_ADDTOPLAYLIST
+            | TRACKMODELCAPS_RELOADMETADATA
+            | TRACKMODELCAPS_LOADTODECK
+            | TRACKMODELCAPS_LOADTOSAMPLER
+            | TRACKMODELCAPS_REMOVE;
 
     // Only allow Add to AutoDJ if we aren't currently showing the AutoDJ queue.
-    if (m_iPlaylistId != m_playlistDao.getPlaylistIdFromName(AUTODJ_TABLE))
+    if (m_iPlaylistId != m_playlistDao.getPlaylistIdFromName(AUTODJ_TABLE)) {
         caps |= TRACKMODELCAPS_ADDTOAUTODJ;
+    }
+
+    bool locked = m_playlistDao.isPlaylistLocked(m_iPlaylistId);
+
+    if (locked) {
+        caps |= TRACKMODELCAPS_LOCKED;
+    }
 
     return caps;
 }
