@@ -86,6 +86,7 @@ void MessageHandler(QtMsgType type, const char *input)
     const char* s = ba.constData();
 
 
+
     if(!Logfile.isOpen())
     {
         QString logFileName = QDir::homePath().append("/").append(SETTINGS_PATH).append("/mixxx.log");
@@ -134,12 +135,16 @@ void MessageHandler(QtMsgType type, const char *input)
     Logfile.flush();
 }
 
-
 int main(int argc, char * argv[])
 {
     // Check if an instance of Mixxx is already running
     // See http://qt.nokia.com/products/appdev/add-on-products/catalog/4/Utilities/qtsingleapplication
 
+    // These need to be set early on (not sure how early) in order to trigger
+    // logic in the OS X appstore support patch from QTBUG-16549.
+    QCoreApplication::setOrganizationDomain("mixxx.org");
+    QCoreApplication::setOrganizationName("Mixxx");
+
 //it seems like this code should be inline in MessageHandler() but for some reason having it there corrupts the messages sometimes -kousu 2/2009
 
 
@@ -154,14 +159,7 @@ int main(int argc, char * argv[])
     //  so if you change it here, change it also in:
     //      * ErrorDialogHandler::errorDialog()
     QThread::currentThread()->setObjectName("Main");
-    a = new QApplication(argc, argv);
-
-    // Load the translations for Qt and for Mixxx
-
-    QTranslator* qtTranslator = new QTranslator();
-    qtTranslator->load("qt_" + QLocale::system().name(),
-                      QLibraryInfo::location(QLibraryInfo::TranslationsPath));
-    a->installTranslator(qtTranslator);
+    QApplication a(argc, argv);
 
     //Enumerate and load SoundSource plugins
     SoundSourceProxy::loadPlugins();
@@ -169,13 +167,6 @@ int main(int argc, char * argv[])
     //LADSPALoader ladspaloader;
 #endif
 
-    QTranslator tor(0);
-    // set the location where your .qm files are in load() below as the last parameter instead of "."
-    // for development, use "/" to use the english original as
-    // .qm files are stored in the base project directory.
-    tor.load(QString("mixxx.") + QLocale::system().name(), ".");
-    a->installTranslator(&tor);
-
     // Check if one of the command line arguments is "--no-visuals"
 //    bool bVisuals = true;
 //    for (int i=0; i<argc; ++i)
@@ -189,22 +180,25 @@ int main(int argc, char * argv[])
     // Only match supported file types since command line options are also parsed elsewhere
     QRegExp fileRx(SoundSourceProxy::supportedFileExtensionsRegex(), Qt::CaseInsensitive);
 
-    for (int i=0; i<argc; ++i)
-    {
-        if (argv[i]==QString("-h") || argv[i]==QString("--h") || argv[i]==QString("--help")) {
-            printf("Mixxx digital DJ software v");
-            printf(VERSION);
-            printf(" - Command line options");
-            printf("\n(These are case-sensitive.)\n\n\
+   for (int i = 0; i < argc; ++i) {
+       if (   argv[i] == QString("-h")
+            || argv[i] == QString("--h")
+            || argv[i] == QString("--help")
+    ) {
+           puts("Mixxx digital DJ software v");
+           puts(VERSION);
+           puts(" - Command line options");
+           puts(
+                   "\n(These are case-sensitive.)\n\n\
     [FILE]                  Load the specified music file(s) at start-up.\n\
                             Each must be one of the following file types:\n\
                             ");
 
             QString fileExtensions = SoundSourceProxy::supportedFileExtensionsString();
             QByteArray fileExtensionsBA = QString(fileExtensions).toUtf8();
-            printf(fileExtensionsBA);
-            printf("\n\n");
-            printf("\
+            puts(fileExtensionsBA.constData());
+            puts("\n\n");
+            puts("\
                             Each file you specify will be loaded into the\n\
                             next virtual deck.\n\
 \n\
@@ -218,24 +212,28 @@ int main(int argc, char * argv[])
 \n\
     --midiDebug             Causes Mixxx to display/log all of the MIDI\n\
                             messages it receives and script functions it loads\n\
+\n\
+    --locale LOCALE         Use a custom locale for loading translations\n\
+                            (e.g 'fr')\n\
 \n\
     -f, --fullScreen        Starts Mixxx in full-screen mode\n\
 \n\
     -h, --help              Display this help message and exit");
 
-            printf("\n\n(For more information, see http://mixxx.org/wiki/doku.php/command_line_options)\n");
+            puts("\n\n(For more information, see http://mixxx.org/wiki/doku.php/command_line_options)\n");
             return(0);
         }
 
         if (argv[i]==QString("-f").toLower() || argv[i]==QString("--f") || argv[i]==QString("--fullScreen"))
         {
             args.bStartInFullscreen = true;
-        }
-        else if (fileRx.indexIn(argv[i]) != -1)
+        } else if (argv[i] == QString("--locale") && i+1 < argc) {
+            args.locale = argv[i+1];
+        } else if (fileRx.indexIn(argv[i]) != -1) {
             args.qlMusicFiles += argv[i];
+        }
     }
 
-
     // set up the plugin paths...
     /*
     qDebug() << "Setting up plugin paths...";
@@ -258,7 +256,7 @@ int main(int argc, char * argv[])
         plugin_paths.push_back ("/usr/lib/ladspa/");
         plugin_paths.push_back ("/usr/lib64/ladspa/");
 #elif __APPLE__
-      QDir dir(a->applicationDirPath());
+      QDir dir(a.applicationDirPath());
      dir.cdUp();
      dir.cd("PlugIns");
          plugin_paths.push_back ("/Library/Audio/Plug-ins/LADSPA");
@@ -275,23 +273,26 @@ int main(int argc, char * argv[])
 
 
 #ifdef __APPLE__
-     qDebug() << "setting Qt's plugin seach path (on OS X)";
      QDir dir(QApplication::applicationDirPath());
-     //Set the search path for Qt plugins to be in the bundle's PlugIns directory,
-     //but only if we think the mixxx binary is in a bundle.
-     if (dir.path().contains("Mixxx.app")) {
-        dir.cdUp();
-        dir.cd("PlugIns");
-        //For some reason we need to do setLibraryPaths() and not addLibraryPath().
-        //The latter causes weird problems once the binary is bundled (happened with 1.7.2 when Brian packaged it up).
-        QApplication::setLibraryPaths(QStringList(dir.absolutePath()));
+     // Set the search path for Qt plugins to be in the bundle's PlugIns
+     // directory, but only if we think the mixxx binary is in a bundle.
+     if (dir.path().contains(".app/")) {
+         // If in a bundle, applicationDirPath() returns something formatted
+         // like: .../Mixxx.app/Contents/MacOS
+         dir.cdUp();
+         dir.cd("PlugIns");
+         qDebug() << "Setting Qt plugin search path to:" << dir.absolutePath();
+         // asantoni: For some reason we need to do setLibraryPaths() and not
+         // addLibraryPath(). The latter causes weird problems once the binary
+         // is bundled (happened with 1.7.2 when Brian packaged it up).
+         QApplication::setLibraryPaths(QStringList(dir.absolutePath()));
      }
 #endif
 
-    MixxxApp *mixxx = new MixxxApp(a, args);
+    MixxxApp* mixxx = new MixxxApp(&a, args);
 
-    //a->setMainWidget(mixxx);
-    a->connect(a, SIGNAL(lastWindowClosed()), a, SLOT(quit()));
+    //a.setMainWidget(mixxx);
+    QObject::connect(&a, SIGNAL(lastWindowClosed()), &a, SLOT(quit()));
 
     int result = -1;
 
@@ -300,7 +301,7 @@ int main(int argc, char * argv[])
         mixxx->show();
 
         qDebug() << "Running Mixxx";
-        result = a->exec();
+        result = a.exec();
     }
 
     delete mixxx;
