@@ -33,13 +33,14 @@
 #include "widget/wvumeter.h"
 #include "widget/wstatuslight.h"
 #include "widget/wlabel.h"
+#include "widget/wtime.h"
 #include "widget/wtracktext.h"
 #include "widget/wtrackproperty.h"
 #include "widget/wnumber.h"
 #include "widget/wnumberpos.h"
-#include "widget/wnumberbpm.h"
 #include "widget/wnumberrate.h"
 #include "widget/woverview.h"
+#include "widget/wspinny.h"
 
 #include "widget/wvisualsimple.h"
 #include "widget/wglwaveformviewer.h"
@@ -58,11 +59,13 @@ QMutex LegacySkinParser::s_safeStringMutex;
 LegacySkinParser::LegacySkinParser(ConfigObject<ConfigValue>* pConfig,
                                    MixxxKeyboard* pKeyboard,
                                    PlayerManager* pPlayerManager,
-                                   Library* pLibrary)
+                                   Library* pLibrary,
+                                   VinylControlManager* pVCMan)
         : m_pConfig(pConfig),
           m_pKeyboard(pKeyboard),
           m_pPlayerManager(pPlayerManager),
           m_pLibrary(pLibrary),
+          m_pVCManager(pVCMan),
           m_pParent(NULL) {
 
 }
@@ -252,9 +255,8 @@ QWidget* LegacySkinParser::parseNode(QDomElement node, QWidget *pGrandparent) {
         return parseNumberRate(node);
     } else if (nodeName == "NumberPos") {
         return parseNumberPos(node);
-    } else if (nodeName == "NumberBpm") {
-        return parseNumberBpm(node);
-    } else if (nodeName == "Number") {
+    } else if (nodeName == "Number" || nodeName == "NumberBpm") {
+        // NumberBpm is deprecated, and is now the same as a Number
         return parseNumber(node);
     } else if (nodeName == "Label") {
         return parseLabel(node);
@@ -266,6 +268,10 @@ QWidget* LegacySkinParser::parseNode(QDomElement node, QWidget *pGrandparent) {
         return parseWidgetGroup(node);
     } else if (nodeName == "Style") {
         return parseStyle(node);
+    } else if (nodeName == "Spinny") {
+        return parseSpinny(node);
+    } else if (nodeName == "Time") {
+        return parseTime(node);
     } else {
         qDebug() << "Invalid node name in skin:" << nodeName;
     }
@@ -407,13 +413,21 @@ QWidget* LegacySkinParser::parseOverview(QDomElement node) {
 
     // Connect the player's load and unload signals to the overview widget.
     connect(pPlayer, SIGNAL(newTrackLoaded(TrackPointer)),
-            p, SLOT(slotLoadNewWaveform(TrackPointer)));
+            p, SLOT(slotTrackLoaded(TrackPointer)));
     connect(pPlayer, SIGNAL(unloadingTrack(TrackPointer)),
             p, SLOT(slotUnloadTrack(TrackPointer)));
 
+    // Connect the load progress of waveforms signals so that we can use this
+    // widget as a progress bar.
+    AnalyserQueue* pAnalyserQueue = pPlayer->getAnalyserQueue();
+    if (pAnalyserQueue) {
+        connect(pAnalyserQueue, SIGNAL(trackProgress(TrackPointer, int)),
+                p, SLOT(slotTrackProgress(TrackPointer, int)));
+    }
+
     TrackPointer pTrack = pPlayer->getLoadedTrack();
     if (pTrack) {
-        p->slotLoadNewWaveform(pTrack);
+        p->slotTrackLoaded(pTrack);
     }
 
     return p;
@@ -437,11 +451,13 @@ QWidget* LegacySkinParser::parseVisual(QDomElement node) {
     widget->installEventFilter(m_pKeyboard);
 
     // Hook up the wheel Control Object to the Visual Controller
+
+    // Connect control proxy to widget, so delete can be handled by the QT object tree
     ControlObjectThreadWidget * p = new ControlObjectThreadWidget(
-        ControlObject::getControl(ConfigKey(channelStr, "wheel")));
+        ControlObject::getControl(ConfigKey(channelStr, "wheel")), widget);
 
     p->setWidget((QWidget *)widget, true, true,
-                 ControlObjectThreadWidget::EMIT_ON_PRESS, Qt::LeftButton);
+                 ControlObjectThreadWidget::EMIT_ON_PRESS, Qt::RightButton);
 
     setupWidget(node, widget);
     if (type == WAVEFORM_GL) {
@@ -579,35 +595,6 @@ QWidget* LegacySkinParser::parseNumberPos(QDomElement node) {
     return p;
 }
 
-QWidget* LegacySkinParser::parseNumberBpm(QDomElement node) {
-    QString channelStr = lookupNodeGroup(node);
-
-    const char* pSafeChannelStr = safeChannelString(channelStr);
-
-    BaseTrackPlayer* pPlayer = m_pPlayerManager->getPlayer(channelStr);
-
-    if (!pPlayer)
-        return NULL;
-
-    WNumberBpm * p = new WNumberBpm(pSafeChannelStr, m_pParent);
-    setupWidget(node, p);
-    p->setup(node);
-    setupConnections(node, p);
-    p->installEventFilter(m_pKeyboard);
-
-    connect(pPlayer, SIGNAL(newTrackLoaded(TrackPointer)),
-            p, SLOT(slotTrackLoaded(TrackPointer)));
-    connect(pPlayer, SIGNAL(unloadingTrack(TrackPointer)),
-            p, SLOT(slotTrackUnloaded(TrackPointer)));
-
-    TrackPointer pTrack = pPlayer->getLoadedTrack();
-    if (pTrack) {
-        p->slotTrackLoaded(pTrack);
-    }
-
-    return p;
-}
-
 QWidget* LegacySkinParser::parseNumber(QDomElement node) {
     WNumber* p = new WNumber(m_pParent);
     setupWidget(node, p);
@@ -626,6 +613,15 @@ QWidget* LegacySkinParser::parseLabel(QDomElement node) {
     return p;
 }
 
+QWidget* LegacySkinParser::parseTime(QDomElement node) {
+   WTime *p = new WTime(m_pParent);
+   setupWidget(node, p);
+   p->setup(node);
+   setupConnections(node, p);
+   p->installEventFilter(m_pKeyboard);
+   return p;
+}
+
 QWidget* LegacySkinParser::parseKnob(QDomElement node) {
     WKnob * p = new WKnob(m_pParent);
     setupWidget(node, p);
@@ -636,12 +632,36 @@ QWidget* LegacySkinParser::parseKnob(QDomElement node) {
     return p;
 }
 
+QWidget* LegacySkinParser::parseSpinny(QDomElement node) {
+    QString channelStr = lookupNodeGroup(node);
+    const char* pSafeChannelStr = safeChannelString(channelStr);
+    WSpinny* p = new WSpinny(m_pParent, m_pVCManager);
+    setupWidget(node, p);
+
+    connect(p, SIGNAL(trackDropped(QString, QString)),
+            m_pPlayerManager, SLOT(slotLoadToPlayer(QString, QString)));
+
+    p->setup(node, pSafeChannelStr);
+    setupConnections(node, p);
+    p->installEventFilter(m_pKeyboard);
+    return p;
+}
+
+
 QWidget* LegacySkinParser::parseTableView(QDomElement node) {
     QStackedWidget* pTabWidget = new QStackedWidget(m_pParent);
 
     setupPosition(node, pTabWidget);
     setupSize(node, pTabWidget);
 
+    // set maximum width to prevent growing to qSplitter->sizeHint()
+    // Note: sizeHint() may be greater in skins for tiny screens
+    int width = pTabWidget->minimumWidth();
+    if (width == 0) {
+        width = m_pParent->minimumWidth();
+    }
+    pTabWidget->setMaximumWidth(width);
+
     QWidget* pLibraryPage = new QWidget(pTabWidget);
 
     QGridLayout* pLibraryPageLayout = new QGridLayout(pLibraryPage);
@@ -945,8 +965,9 @@ void LegacySkinParser::setupConnections(QDomNode node, QWidget* pWidget) {
             new PropertyBinder(pWidget, property, control);
         } else if (!XmlParse::selectNode(con, "OnOff").isNull() &&
             XmlParse::selectNodeQString(con, "OnOff")=="true") {
-            // Connect control proxy to widget
-            (new ControlObjectThreadWidget(control))->setWidgetOnOff(pWidget);
+            // Connect control proxy to widget. Parented to pWidget so it is not
+            // leaked.
+            (new ControlObjectThreadWidget(control, pWidget))->setWidgetOnOff(pWidget);
         } else {
             // Default to emit on press
             ControlObjectThreadWidget::EmitOption emitOption = ControlObjectThreadWidget::EMIT_ON_PRESS;
@@ -975,8 +996,9 @@ void LegacySkinParser::setupConnections(QDomNode node, QWidget* pWidget) {
                     state = Qt::RightButton;
             }
 
-            // Connect control proxy to widget
-            (new ControlObjectThreadWidget(control))->setWidget(
+            // Connect control proxy to widget. Parented to pWidget so it is not
+            // leaked.
+            (new ControlObjectThreadWidget(control, pWidget))->setWidget(
                 pWidget, connectValueFromWidget, connectValueToWidget,
                 emitOption, state);
 
