@@ -26,10 +26,10 @@
 #include "sounddevice.h"
 #include "sounddeviceportaudio.h"
 #include "engine/enginemaster.h"
+#include "engine/enginebuffer.h"
 #include "controlobjectthreadmain.h"
-#include "audiopath.h"
+#include "soundmanagerutil.h"
 #include "controlobject.h"
-#include "vinylcontrolproxy.h"
 
 #ifdef __PORTAUDIO__
 typedef PaError (*SetJackClientName)(const char *name);
@@ -48,27 +48,20 @@ SoundManager::SoundManager(ConfigObject<ConfigValue> *pConfig, EngineMaster *pMa
     , m_inputDevicesOpened(0)
     , m_pErrorDevice(NULL)
 #ifdef __PORTAUDIO__
-    , m_paInitialized(false)
-    , m_jackSampleRate(-1)
+   , m_paInitialized(false)
+   , m_jackSampleRate(-1)
 #endif
 {
     //These are ControlObjectThreadMains because all the code that
     //uses them is called from the GUI thread (stuff like opening soundcards).
-    ControlObjectThreadMain* pControlObjectLatency = new ControlObjectThreadMain(ControlObject::getControl(ConfigKey("[Master]", "latency")));
-    ControlObjectThreadMain* pControlObjectSampleRate = new ControlObjectThreadMain(ControlObject::getControl(ConfigKey("[Master]", "samplerate")));
-    ControlObjectThreadMain* pControlObjectVinylControlMode = new ControlObjectThreadMain(new ControlObject(ConfigKey("[VinylControl]", "Mode")));
-    ControlObjectThreadMain* pControlObjectVinylControlEnabled = new ControlObjectThreadMain(new ControlObject(ConfigKey("[VinylControl]", "Enabled")));
-    ControlObjectThreadMain* pControlObjectVinylControlGain = new ControlObjectThreadMain(new ControlObject(ConfigKey("[VinylControl]", "VinylControlGain")));
-    ControlObjectThreadMain* pControlObjectVinylControlSignalQuality1 = new ControlObjectThreadMain(new ControlObject(ConfigKey("[Channel1]", "VinylControlQuality")));
-    ControlObjectThreadMain* pControlObjectVinylControlSignalQuality2 = new ControlObjectThreadMain(new ControlObject(ConfigKey("[Channel2]", "VinylControlQuality")));
-    ControlObjectThreadMain* pControlObjectVinylControlInputStrengthL1 = new ControlObjectThreadMain(new ControlObject(ConfigKey("[Channel1]", "VinylControlInputL")));
-    ControlObjectThreadMain* pControlObjectVinylControlInputStrengthR1 = new ControlObjectThreadMain(new ControlObject(ConfigKey("[Channel1]", "VinylControlInputR")));
-    ControlObjectThreadMain* pControlObjectVinylControlInputStrengthL2 = new ControlObjectThreadMain(new ControlObject(ConfigKey("[Channel2]", "VinylControlInputL")));
-    ControlObjectThreadMain* pControlObjectVinylControlInputStrengthR2 = new ControlObjectThreadMain(new ControlObject(ConfigKey("[Channel2]", "VinylControlInputR")));
-
-    pControlObjectVinylControlMode->slotSet(m_pConfig->getValueString(ConfigKey("[VinylControl]","Mode")).toInt());
-    pControlObjectVinylControlEnabled->slotSet(m_pConfig->getValueString(ConfigKey("[VinylControl]","Enabled")).toInt());
-    pControlObjectVinylControlGain->slotSet(m_pConfig->getValueString(ConfigKey("[VinylControl]","VinylControlGain")).toInt());
+    // TODO(xxx) some of these ControlObject are not needed by soundmanager, or are unused here.
+    // It is possible to take them out?    
+    m_pControlObjectLatency = new ControlObjectThreadMain(ControlObject::getControl(ConfigKey("[Master]", "latency")));
+    m_pControlObjectSampleRate = new ControlObjectThreadMain(ControlObject::getControl(ConfigKey("[Master]", "samplerate")));
+    m_pControlObjectVinylControlMode = new ControlObjectThreadMain(new ControlObject(ConfigKey("[VinylControl]", "mode")));
+    m_pControlObjectVinylControlMode1 = new ControlObjectThreadMain(ControlObject::getControl(ConfigKey("[Channel1]", "vinylcontrol_mode")));
+    m_pControlObjectVinylControlMode2 = new ControlObjectThreadMain(ControlObject::getControl(ConfigKey("[Channel2]", "vinylcontrol_mode")));
+    m_pControlObjectVinylControlGain = new ControlObjectThreadMain(new ControlObject(ConfigKey("[VinylControl]", "gain")));
 
     //Hack because PortAudio samplerate enumeration is slow as hell on Linux (ALSA dmix sucks, so we can't blame PortAudio)
     m_samplerates.push_back(44100);
@@ -86,8 +79,9 @@ SoundManager::SoundManager(ConfigObject<ConfigValue> *pConfig, EngineMaster *pMa
 
     // TODO(bkgood) do these really need to be here? they're set in
     // SoundDevicePortAudio::open
-    pControlObjectLatency->slotSet(m_config.getFramesPerBuffer() / m_config.getSampleRate() * 1000);
-    pControlObjectSampleRate->slotSet(m_config.getSampleRate());
+    m_pControlObjectLatency->slotSet(
+        m_config.getFramesPerBuffer() / m_config.getSampleRate() * 1000);
+    m_pControlObjectSampleRate->slotSet(m_config.getSampleRate());
 }
 
 /** Destructor for the SoundManager class. Closes all the devices, cleans up their pointers
@@ -97,12 +91,21 @@ SoundManager::~SoundManager()
     //Clean up devices.
     clearDeviceList();
 
+#ifdef __PORTAUDIO__
     if (m_paInitialized) {
         Pa_Terminate();
         m_paInitialized = false;
     }
+#endif
     // vinyl control proxies and input buffers are freed in closeDevices, called
     // by clearDeviceList -- bkgood
+
+    delete m_pControlObjectLatency;
+    delete m_pControlObjectSampleRate;
+    delete m_pControlObjectVinylControlMode;
+    delete m_pControlObjectVinylControlMode1;
+    delete m_pControlObjectVinylControlMode2;
+    delete m_pControlObjectVinylControlGain;
 }
 
 /**
@@ -232,6 +235,12 @@ void SoundManager::closeDevices()
                              // pointers to memory owned by EngineMaster)
 
     foreach (AudioInput in, m_inputBuffers.keys()) {
+        // Need to tell all registered AudioDestinations for this AudioInput
+        // that the input was disconnected.
+        if (m_registeredDestinations.contains(in)) {
+            m_registeredDestinations[in]->onInputDisconnected(in);
+        }
+
         short *buffer = m_inputBuffers[in];
         if (buffer != NULL) {
             delete [] buffer;
@@ -239,18 +248,6 @@ void SoundManager::closeDevices()
         }
     }
     m_inputBuffers.clear();
-
-#ifdef __VINYLCONTROL__
-    // TODO(bkgood) see comment where these objects are created in setupDevices,
-    // this should probably be in the dtor or at least somewhere other
-    // than here.
-    while (!m_vinylControl.empty()) {
-        VinylControlProxy *vc = m_vinylControl.takeLast();
-        if (vc != NULL) {
-            delete vc;
-        }
-    }
-#endif
 }
 
 /** Closes all the devices and empties the list of devices we have. */
@@ -372,6 +369,8 @@ int SoundManager::setupDevices()
     clearOperativeVariables();
     int devicesAttempted = 0;
     int devicesOpened = 0;
+    // pair is isInput, isOutput
+    QHash<SoundDevice*, QPair<bool, bool> > toOpen;
 
     // filter out any devices in the config we don't actually have
     m_config.filterOutputs(this);
@@ -379,14 +378,6 @@ int SoundManager::setupDevices()
 
     // close open devices, close running vinyl control proxies
     closeDevices();
-#ifdef __VINYLCONTROL__
-    //Initialize vinyl control
-    // TODO(bkgood) this ought to be done in the ctor or something. Not here. Really
-    // shouldn't be any reason for these to be reinitialized every time the
-    // audio prefs are updated. Will require work in DlgPrefVinyl.
-    m_vinylControl.append(new VinylControlProxy(m_pConfig, "[Channel1]"));
-    m_vinylControl.append(new VinylControlProxy(m_pConfig, "[Channel2]"));
-#endif
     foreach (SoundDevice *device, m_devices) {
         bool isInput = false;
         bool isOutput = false;
@@ -396,62 +387,58 @@ int SoundManager::setupDevices()
         foreach (AudioInput in, m_config.getInputs().values(device->getInternalName())) {
             isInput = true;
             err = device->addInput(in);
-            if (err != OK)
-                return err;
+            if (err != OK) goto closeAndError;
             if (!m_inputBuffers.contains(in)) {
                 // TODO(bkgood) look into allocating this with the frames per
                 // buffer value from SMConfig
                 m_inputBuffers[in] = new short[MAX_BUFFER_LEN];
             }
+
+            // Check if any AudioDestination is registered for this AudioInput,
+            // and call the onInputConnected method.
+            if (m_registeredDestinations.contains(in)) {
+                m_registeredDestinations[in]->onInputConnected(in);
+            }
         }
         foreach (AudioOutput out, m_config.getOutputs().values(device->getInternalName())) {
             isOutput = true;
             // following keeps us from asking for a channel buffer EngineMaster
             // doesn't have -- bkgood
-            if (out.getType() == AudioOutput::DECK
-                    && out.getIndex() >= m_pMaster->numChannels()) continue;
+            if (m_registeredSources[out]->buffer(out) == NULL) {
+                qDebug() << "AudioSource returned null for" << out.getString();
+                continue;
+            }
             err = device->addOutput(out);
-            if (err != OK)
-                return err;
-            // TODO(bkgood) this would be nicer as something like
-            // EngineMaster::getBuffer(AudioPathType type, uint index = 0);
-            // but I don't want to mess with enginemaster if I can help it
-            // before hydra's merged
-            switch (out.getType()) {
-            case AudioPath::MASTER:
-                m_outputBuffers[out] = m_pMaster->getMasterBuffer();
-                // Set the master output device's clock as the reference
-                //  to avoid any clock-related disturbances to the public address
+            if (err != OK) goto closeAndError;
+            m_outputBuffers[out] = m_registeredSources[out]->buffer(out);
+            if (out.getType() == AudioOutput::MASTER) {
+                m_pClkRefDevice = device;
+            } else if (out.getType() == AudioOutput::DECK
+                    && !m_pClkRefDevice) {
                 m_pClkRefDevice = device;
-                break;
-            case AudioPath::HEADPHONES:
-                m_outputBuffers[out] = m_pMaster->getHeadphoneBuffer();
-                break;
-            case AudioPath::DECK:
-                m_outputBuffers[out] = m_pMaster->getChannelBuffer(out.getIndex());
-                // If a reference device has not yet been set (such as when the Master output
-                //  isn't being used with an external mixer,) use the first deck output
-                //  to avoid any clock-related disturbances on at least one public output
-                if (!m_pClkRefDevice) m_pClkRefDevice = device;
-                break;
-            default:
-                break;
             }
         }
         if (isInput || isOutput) {
             device->setSampleRate(m_config.getSampleRate());
             device->setFramesPerBuffer(m_config.getFramesPerBuffer());
-            ++devicesAttempted;
-            err = device->open();
-            if (err != OK) {
-                return err;
-            } else {
-                ++devicesOpened;
-                if (isOutput)
-                    ++m_outputDevicesOpened;
-                if (isInput)
-                    ++m_inputDevicesOpened;
-            }
+            toOpen[device] = QPair<bool, bool>(isInput, isOutput);
+        }
+    }
+    foreach (SoundDevice *device, toOpen.keys()) {
+        QPair<bool, bool> mode(toOpen[device]);
+        bool isInput(mode.first);
+        bool isOutput(mode.second);
+        ++devicesAttempted;
+        m_pErrorDevice = device;
+        err = device->open();
+        if (err != OK) {
+            goto closeAndError;
+        } else {
+            ++devicesOpened;
+            if (isOutput)
+                ++m_outputDevicesOpened;
+            if (isInput)
+                ++m_inputDevicesOpened;
         }
     }
 
@@ -470,15 +457,19 @@ int SoundManager::setupDevices()
     }
 
     qDebug() << m_outputDevicesOpened << "output sound devices opened";
-    qDebug() << m_inputDevicesOpened << "input sound devices opened";
+    qDebug() << m_inputDevicesOpened << "input  sound devices opened";
 
     // returns OK if we were able to open all the devices the user
     // wanted
     if (devicesAttempted == devicesOpened) {
+        emit(devicesSetup());
         return OK;
     }
     m_pErrorDevice = NULL;
     return ERR;
+closeAndError:
+    closeDevices();
+    return err;
 }
 
 SoundDevice* SoundManager::getErrorDevice() const {
@@ -493,35 +484,31 @@ int SoundManager::setConfig(SoundManagerConfig config) {
     int err = OK;
     m_config = config;
     checkConfig();
+
+    // certain parts of mixxx rely on this being here, for the time being, just
+    // letting those be -- bkgood
+    // Do this first so vinyl control gets the right samplerate -- Owen W.
+    m_pConfig->set(ConfigKey("[Soundcard]","Samplerate"), ConfigValue(m_config.getSampleRate()));
+
     err = setupDevices();
     if (err == OK) {
         m_config.writeToDisk();
     }
-    // certain parts of mixxx rely on this being here, for the time being, just
-    // letting those be -- bkgood
-    m_pConfig->set(ConfigKey("[Soundcard]","Samplerate"), ConfigValue(m_config.getSampleRate()));
     return err;
 }
 
 void SoundManager::checkConfig() {
     if (!m_config.checkAPI(*this)) {
-        m_config.setAPI(DEFAULT_API);
+        m_config.setAPI(SoundManagerConfig::kDefaultAPI);
         m_config.loadDefaults(this, SoundManagerConfig::API | SoundManagerConfig::DEVICES);
     }
     if (!m_config.checkSampleRate(*this)) {
-        m_config.setSampleRate(DEFAULT_SAMPLE_RATE);
+        m_config.setSampleRate(SoundManagerConfig::kDefaultSampleRate);
         m_config.loadDefaults(this, SoundManagerConfig::OTHER);
     }
     // latency checks itself for validity on SMConfig::setLatency()
 }
 
-void SoundManager::sync()
-{
-    ControlObject::sync();
-    //qDebug() << "sync";
-
-}
-
 //Requests a buffer in the proper format, if we're prepared to give one.
 QHash<AudioOutput, const CSAMPLE*>
 SoundManager::requestBuffer(QList<AudioOutput> outputs,
@@ -565,8 +552,6 @@ SoundManager::requestBuffer(QList<AudioOutput> outputs,
     {
         // Only generate a new buffer for the clock reference card
 //         qDebug() << "New buffer for" << device->getDisplayName() << "of size" << iFramesPerBuffer;
-        //First, sync control parameters with changes from GUI thread
-        sync();
 
         //Process a block of samples for output. iFramesPerBuffer is the
         //number of samples for one channel, but the EngineObject
@@ -583,6 +568,8 @@ SoundManager::requestBuffer(QList<AudioOutput> outputs,
 void SoundManager::pushBuffer(QList<AudioInput> inputs, short * inputBuffer,
                               unsigned long iFramesPerBuffer, unsigned int iFrameSize)
 {
+    //This function is called a *lot* and is a big source of CPU usage.
+    //It needs to be very fast.
 
 //    m_inputBuffers[RECEIVER_VINYLCONTROL_ONE]
 
@@ -664,26 +651,56 @@ void SoundManager::pushBuffer(QList<AudioInput> inputs, short * inputBuffer,
 
     if (inputBuffer)
     {
-#ifdef __VINYLCONTROL__
-        for (QList<AudioInput>::const_iterator i = inputs.begin(),
+        for (QList<AudioInput>::ConstIterator i = inputs.begin(),
                      e = inputs.end(); i != e; ++i) {
             const AudioInput& in = *i;
-            if (in.getType() == AudioInput::VINYLCONTROL) {
-                unsigned int index = in.getIndex();
-                Q_ASSERT(index < 2); // XXX we only do two vc decks atm -- bkgood
-                if (m_vinylControl[index] && m_inputBuffers.contains(in)) {
-                    m_vinylControl[index]->AnalyseSamples(m_inputBuffers[in], iFramesPerBuffer);
+
+            // Sanity check.
+            if (!m_inputBuffers.contains(in)) {
+                continue;
+            }
+
+            short* pInputBuffer = m_inputBuffers[in];
+
+            if (m_registeredDestinations.contains(in)) {
+                AudioDestination* destination = m_registeredDestinations[in];
+                if (destination) {
+                    destination->receiveBuffer(in, pInputBuffer, iFramesPerBuffer);
                 }
             }
         }
-#endif
     }
     //TODO: Add pass-through option here (and push it into EngineMaster)...
     //      (or maybe save it, and then have requestBuffer() push it into EngineMaster)...
 }
 
-void SoundManager::setJACKName() const
-{
+void SoundManager::registerOutput(AudioOutput output, const AudioSource *src) {
+    if (m_registeredSources.contains(output)) {
+        qDebug() << "WARNING: AudioOutput already registered!";
+    }
+    m_registeredSources[output] = src;
+    emit(outputRegistered(output, src));
+}
+
+void SoundManager::registerInput(AudioInput input, AudioDestination *dest) {
+    if (m_registeredDestinations.contains(input)) {
+        // note that this can be totally ok if we just want a certain
+        // AudioInput to be going to a different AudioDest -bkgood
+        qDebug() << "WARNING: AudioInput already registered!";
+    }
+    m_registeredDestinations[input] = dest;
+    emit(inputRegistered(input, dest));
+}
+
+QList<AudioOutput> SoundManager::registeredOutputs() const {
+    return m_registeredSources.keys();
+}
+
+QList<AudioInput> SoundManager::registeredInputs() const {
+    return m_registeredDestinations.keys();
+}
+
+void SoundManager::setJACKName() const {
 #ifdef __PORTAUDIO__
 #ifdef Q_OS_LINUX
     typedef PaError (*SetJackClientName)(const char *name);
