@@ -21,12 +21,14 @@
 #include "defs.h"
 #include "configobject.h"
 #include "soundmanagerconfig.h"
+#include "controlobjectthreadmain.h"
 
 class SoundDevice;
 class EngineMaster;
 class AudioOutput;
 class AudioInput;
-class VinylControlProxy;
+class AudioSource;
+class AudioDestination;
 
 #define MIXXX_PORTAUDIO_JACK_STRING "JACK Audio Connection Kit"
 #define MIXXX_PORTAUDIO_ALSA_STRING "ALSA"
@@ -40,7 +42,7 @@ class SoundManager : public QObject {
 public:
     SoundManager(ConfigObject<ConfigValue> *pConfig, EngineMaster *_master);
     ~SoundManager();
-    const EngineMaster* getEngine() const;
+    const EngineMaster* getEngine() const; // this shouldn't exist
     QList<SoundDevice*> getDeviceList(QString filterAPI, bool bOutputDevices, bool bInputDevices);
     void closeDevices();
     void clearDeviceList();
@@ -58,10 +60,15 @@ public:
         SoundDevice *device, double streamTime = 0);
     void pushBuffer(QList<AudioInput> inputs, short *inputBuffer,
         unsigned long iFramesPerBuffer, unsigned int iFrameSize);
+    void registerOutput(AudioOutput output, const AudioSource *src);
+    void registerInput(AudioInput input, AudioDestination *dest);
+    QList<AudioOutput> registeredOutputs() const;
+    QList<AudioInput> registeredInputs() const;
 signals:
     void devicesUpdated(); // emitted when pointers to SoundDevices go stale
-public slots:
-    void sync();
+    void devicesSetup(); // emitted when the sound devices have been set up
+    void outputRegistered(AudioOutput output, const AudioSource *src);
+    void inputRegistered(AudioInput input, AudioDestination *dest);
 private:
     void clearOperativeVariables();
     void setJACKName() const;
@@ -77,9 +84,6 @@ private:
     // Clock reference, used to make sure the same device triggers buffer
     // refresh every $latency-ms period
     SoundDevice* m_pClkRefDevice;
-#ifdef __VINYLCONTROL__
-    QList<VinylControlProxy*> m_vinylControl;
-#endif
     int m_outputDevicesOpened;
     int m_inputDevicesOpened;
     QMutex requestBufferMutex;
@@ -89,6 +93,15 @@ private:
     bool m_paInitialized;
     unsigned int m_jackSampleRate;
 #endif
+    QHash<AudioOutput, const AudioSource*> m_registeredSources;
+    QHash<AudioInput, AudioDestination*> m_registeredDestinations;
+
+    ControlObjectThreadMain* m_pControlObjectLatency;
+    ControlObjectThreadMain* m_pControlObjectSampleRate;
+    ControlObjectThreadMain* m_pControlObjectVinylControlMode;
+    ControlObjectThreadMain* m_pControlObjectVinylControlMode1;
+    ControlObjectThreadMain* m_pControlObjectVinylControlMode2;
+    ControlObjectThreadMain* m_pControlObjectVinylControlGain;
 };
 
 #endif
