@@ -41,13 +41,18 @@ SoundSourceMp3::SoundSourceMp3(QString qFilename) :
 SoundSourceMp3::~SoundSourceMp3()
 {
     mad_stream_finish(Stream);
-    mad_frame_finish(Frame);
-    mad_synth_finish(Synth);
-    m_file.unmap(inputbuf);
-    m_file.close();
+    delete Stream;
 
-    //Don't delete because we're using mmapped i/o now.... right?
-    //delete [] inputbuf;
+    mad_frame_finish(Frame);
+    delete Frame;
+
+    mad_synth_finish(Synth);
+    delete Synth;
+
+    // Unmap inputbuf.
+    m_file.unmap(inputbuf);
+    inputbuf = NULL;
+    m_file.close();
 
     //Free the pointers in our seek list, LIBERATE THEM!!!
     for (int i = 0; i < m_qSeekList.count(); i++)
@@ -548,20 +553,20 @@ unsigned SoundSourceMp3::read(unsigned long samples_wanted, const SAMPLE * _dest
 int SoundSourceMp3::parseHeader()
 {
     setType("mp3");
-
-    #ifdef __WINDOWS__
-		/* From Tobias: A Utf-8 string did not work on my Windows XP (German edition)
-		 * If you try this conversion, f.isValid() will return false in many cases
-		 * and processTaglibFile() will fail
-		 *
-		 * The method toLocal8Bit() returns the local 8-bit representation of the string as a QByteArray.
-		 * The returned byte array is undefined if the string contains characters not supported
-		 * by the local 8-bit encoding.
-		 */
-		TagLib::MPEG::File f(getFilename().toLocal8Bit().constData());
-	#else
-		TagLib::MPEG::File f(getFilename().toUtf8().constData());
-	#endif
+#ifdef __WINDOWS__
+    /* From Tobias: A Utf-8 string did not work on my Windows XP (German edition)
+     * If you try this conversion, f.isValid() will return false in many cases
+     * and processTaglibFile() will fail
+     *
+     * The method toLocal8Bit() returns the local 8-bit representation of the string as a QByteArray.
+     * The returned byte array is undefined if the string contains characters not supported
+     * by the local 8-bit encoding.
+     */
+    QByteArray qBAFilename = m_qFilename.toLocal8Bit();
+#else
+    QByteArray qBAFilename = m_qFilename.toUtf8();
+#endif
+    TagLib::MPEG::File f(qBAFilename.constData());
 
     // Takes care of all the default metadata
     bool result = processTaglibFile(f);
