@@ -80,6 +80,7 @@ WTrackTableView::~WTrackTableView()
         pHeader->saveHeaderState();
     }
 
+    delete m_pReloadMetadataAct;
     delete m_pAutoDJAct;
     delete m_pRemoveAct;
     delete m_pPropertiesAct;
@@ -87,8 +88,9 @@ WTrackTableView::~WTrackTableView()
     delete m_pPlaylistMenu;
     delete m_pCrateMenu;
     //delete m_pRenamePlaylistAct;
-
     delete m_pTrackInfo;
+    delete m_pNumSamplers;
+    delete m_pNumDecks;
 }
 
 void WTrackTableView::loadTrackModel(QAbstractItemModel *model) {
@@ -98,6 +100,18 @@ void WTrackTableView::loadTrackModel(QAbstractItemModel *model) {
 
     Q_ASSERT(model);
     Q_ASSERT(track_model);
+
+    /* If the model has not changed
+     * there's no need to exchange the headers
+     * this will cause a small GUI freeze
+     */
+    if (getTrackModel() == track_model) {
+        // Re-sort the table even if the track model is the same. This triggers
+        // a select() if the table is dirty.
+        doSortByColumn(horizontalHeader()->sortIndicatorSection());
+        return;
+    }
+
     setVisible(false);
 
     // Save the previous track model's header state
@@ -144,11 +158,6 @@ void WTrackTableView::loadTrackModel(QAbstractItemModel *model) {
     header->setClickable(true);
     header->setHighlightSections(true);
     header->setSortIndicatorShown(true);
-    //setSortingEnabled(true);
-    connect(horizontalHeader(), SIGNAL(sortIndicatorChanged(int, Qt::SortOrder)),
-            this, SLOT(doSortByColumn(int)), Qt::AutoConnection);
-
-    doSortByColumn(horizontalHeader()->sortIndicatorSection());
 
     // Initialize all column-specific things
     for (int i = 0; i < model->columnCount(); ++i) {
@@ -178,6 +187,31 @@ void WTrackTableView::loadTrackModel(QAbstractItemModel *model) {
         }
     }
 
+    // NOTE: Should be a UniqueConnection but that requires Qt 4.6
+    connect(horizontalHeader(), SIGNAL(sortIndicatorChanged(int, Qt::SortOrder)),
+            this, SLOT(doSortByColumn(int)), Qt::AutoConnection);
+
+    // Stupid hack that assumes column 0 is never visible, but this is a weak
+    // proxy for "there was a saved column sort order"
+    if (horizontalHeader()->sortIndicatorSection() > 0) {
+        // Sort by the saved sort section and order. This line sorts the
+        // TrackModel and in turn generates a select()
+        horizontalHeader()->setSortIndicator(horizontalHeader()->sortIndicatorSection(),
+                                             horizontalHeader()->sortIndicatorOrder());
+    } else {
+        // No saved order is present. Use the TrackModel's default sort order.
+        int sortColumn = track_model->defaultSortColumn();
+        Qt::SortOrder sortOrder = track_model->defaultSortOrder();
+
+        // If the TrackModel has an invalid or internal column as its default
+        // sort, find the first non-internal column and sort by that.
+        while (sortColumn < 0 || track_model->isColumnInternal(sortColumn)) {
+            sortColumn++;
+        }
+        // This line sorts the TrackModel and in turn generates a select()
+        horizontalHeader()->setSortIndicator(sortColumn, sortOrder);
+    }
+
     // Set up drag and drop behaviour according to whether or not the track
     // model says it supports it.
 
@@ -204,6 +238,14 @@ void WTrackTableView::loadTrackModel(QAbstractItemModel *model) {
     setVisible(true);
 }
 
+void WTrackTableView::disableSorting() {
+    // We have to manually do this because setSortingEnabled(false) does not
+    // properly disconnect the signals for some reason.
+    disconnect(horizontalHeader(), SIGNAL(sortIndicatorChanged(int, Qt::SortOrder)),
+               this, SLOT(doSortByColumn(int)));
+    horizontalHeader()->setSortIndicatorShown(false);
+}
+
 void WTrackTableView::createActions() {
     Q_ASSERT(m_pMenu);
     Q_ASSERT(m_pSamplerMenu);
@@ -216,10 +258,16 @@ void WTrackTableView::createActions() {
 
     m_pAutoDJAct = new QAction(tr("Add to Auto DJ Queue"),this);
     connect(m_pAutoDJAct, SIGNAL(triggered()), this, SLOT(slotSendToAutoDJ()));
+
+    m_pReloadMetadataAct = new QAction(tr("Reload Track Metadata"), this);
+    connect(m_pReloadMetadataAct, SIGNAL(triggered()), this, SLOT(slotReloadTrackMetadata()));
 }
 
-void WTrackTableView::slotMouseDoubleClicked(const QModelIndex &index)
-{
+void WTrackTableView::slotMouseDoubleClicked(const QModelIndex &index) {
+    if (!modelHasCapabilities(TrackModel::TRACKMODELCAPS_LOADTODECK)) {
+        return;
+    }
+
     TrackModel* trackModel = getTrackModel();
     TrackPointer pTrack;
     if (trackModel && (pTrack = trackModel->getTrack(index))) {
@@ -230,11 +278,15 @@ void WTrackTableView::slotMouseDoubleClicked(const QModelIndex &index)
 void WTrackTableView::loadSelectionToGroup(QString group) {
     QModelIndexList indices = selectionModel()->selectedRows();
     if (indices.size() > 0) {
-        bool groupPlaying = ControlObject::getControl(
-            ConfigKey(group, "play"))->get() == 1.0f;
+        // If the track load override is disabled, check to see if a track is
+        // playing before trying to load it
+        if ( !(m_pConfig->getValueString(ConfigKey("[Controls]","AllowTrackLoadToPlayingDeck")).toInt()) ) {
+            bool groupPlaying = ControlObject::getControl(
+                ConfigKey(group, "play"))->get() == 1.0f;
 
-        if (groupPlaying)
-            return;
+            if (groupPlaying)
+                return;
+        }
 
         QModelIndex index = indices.at(0);
         TrackModel* trackModel = getTrackModel();
@@ -307,56 +359,64 @@ void WTrackTableView::contextMenuEvent(QContextMenuEvent * event)
         m_pMenu->addSeparator();
     }
 
-    int iNumDecks = m_pNumDecks->get();
-    if (iNumDecks > 0) {
-        for (int i = 1; i <= iNumDecks; ++i) {
-            QString deckGroup = QString("[Channel%1]").arg(i);
-            bool deckPlaying = ControlObject::getControl(
-                ConfigKey(deckGroup, "play"))->get() == 1.0f;
-            bool deckEnabled = !deckPlaying && oneSongSelected;
-            QAction* pAction = new QAction(tr("Load to Deck %1").arg(i), m_pMenu);
-            pAction->setEnabled(deckEnabled);
-            m_pMenu->addAction(pAction);
-            m_deckMapper.setMapping(pAction, deckGroup);
-            connect(pAction, SIGNAL(triggered()), &m_deckMapper, SLOT(map()));
+    if (modelHasCapabilities(TrackModel::TRACKMODELCAPS_LOADTODECK)) {
+        int iNumDecks = m_pNumDecks->get();
+        if (iNumDecks > 0) {
+            for (int i = 1; i <= iNumDecks; ++i) {
+                QString deckGroup = QString("[Channel%1]").arg(i);
+                bool deckPlaying = ControlObject::getControl(
+                    ConfigKey(deckGroup, "play"))->get() == 1.0f;
+                bool deckEnabled = !deckPlaying && oneSongSelected;
+                QAction* pAction = new QAction(tr("Load to Deck %1").arg(i), m_pMenu);
+                pAction->setEnabled(deckEnabled);
+                m_pMenu->addAction(pAction);
+                m_deckMapper.setMapping(pAction, deckGroup);
+                connect(pAction, SIGNAL(triggered()), &m_deckMapper, SLOT(map()));
+            }
         }
     }
 
-    int iNumSamplers = m_pNumSamplers->get();
-    if (iNumSamplers > 0) {
-        m_pSamplerMenu->clear();
-        for (int i = 1; i <= iNumSamplers; ++i) {
-            QString samplerGroup = QString("[Sampler%1]").arg(i);
-            bool samplerPlaying = ControlObject::getControl(
-                ConfigKey(samplerGroup, "play"))->get() == 1.0f;
-            bool samplerEnabled = !samplerPlaying && oneSongSelected;
-            QAction* pAction = new QAction(tr("Sampler %1").arg(i), m_pSamplerMenu);
-            pAction->setEnabled(samplerEnabled);
-            m_pSamplerMenu->addAction(pAction);
-            m_samplerMapper.setMapping(pAction, samplerGroup);
-            connect(pAction, SIGNAL(triggered()), &m_samplerMapper, SLOT(map()));
+    if (modelHasCapabilities(TrackModel::TRACKMODELCAPS_LOADTOSAMPLER)) {
+        int iNumSamplers = m_pNumSamplers->get();
+        if (iNumSamplers > 0) {
+            m_pSamplerMenu->clear();
+            for (int i = 1; i <= iNumSamplers; ++i) {
+                QString samplerGroup = QString("[Sampler%1]").arg(i);
+                bool samplerPlaying = ControlObject::getControl(
+                    ConfigKey(samplerGroup, "play"))->get() == 1.0f;
+                bool samplerEnabled = !samplerPlaying && oneSongSelected;
+                QAction* pAction = new QAction(tr("Sampler %1").arg(i), m_pSamplerMenu);
+                pAction->setEnabled(samplerEnabled);
+                m_pSamplerMenu->addAction(pAction);
+                m_samplerMapper.setMapping(pAction, samplerGroup);
+                connect(pAction, SIGNAL(triggered()), &m_samplerMapper, SLOT(map()));
+            }
+            m_pMenu->addMenu(m_pSamplerMenu);
         }
-        m_pMenu->addMenu(m_pSamplerMenu);
     }
 
     m_pMenu->addSeparator();
 
     if (modelHasCapabilities(TrackModel::TRACKMODELCAPS_ADDTOPLAYLIST)) {
         m_pPlaylistMenu->clear();
-
         PlaylistDAO& playlistDao = m_pTrackCollection->getPlaylistDAO();
         int numPlaylists = playlistDao.playlistCount();
+
         for (int i = 0; i < numPlaylists; ++i) {
             int iPlaylistId = playlistDao.getPlaylistId(i);
-            if (playlistDao.isHidden(iPlaylistId))
-                continue;
-            QString playlistName = playlistDao.getPlaylistName(iPlaylistId);
-            // No leak because making the menu the parent means they will be
-            // auto-deleted
-            QAction* pAction = new QAction(playlistName, m_pPlaylistMenu);
-            m_pPlaylistMenu->addAction(pAction);
-            m_playlistMapper.setMapping(pAction, iPlaylistId);
-            connect(pAction, SIGNAL(triggered()), &m_playlistMapper, SLOT(map()));
+
+            if (!playlistDao.isHidden(iPlaylistId)) {
+
+                QString playlistName = playlistDao.getPlaylistName(iPlaylistId);
+                // No leak because making the menu the parent means they will be
+                // auto-deleted
+                QAction* pAction = new QAction(playlistName, m_pPlaylistMenu);
+                bool locked = playlistDao.isPlaylistLocked(iPlaylistId);
+                pAction->setEnabled(!locked);
+                m_pPlaylistMenu->addAction(pAction);
+                m_playlistMapper.setMapping(pAction, iPlaylistId);
+                connect(pAction, SIGNAL(triggered()), &m_playlistMapper, SLOT(map()));
+            }
         }
 
         m_pMenu->addMenu(m_pPlaylistMenu);
@@ -364,7 +424,6 @@ void WTrackTableView::contextMenuEvent(QContextMenuEvent * event)
 
     if (modelHasCapabilities(TrackModel::TRACKMODELCAPS_ADDTOCRATE)) {
         m_pCrateMenu->clear();
-
         CrateDAO& crateDao = m_pTrackCollection->getCrateDAO();
         int numCrates = crateDao.crateCount();
         for (int i = 0; i < numCrates; ++i) {
@@ -372,6 +431,8 @@ void WTrackTableView::contextMenuEvent(QContextMenuEvent * event)
             // No leak because making the menu the parent means they will be
             // auto-deleted
             QAction* pAction = new QAction(crateDao.crateName(iCrateId), m_pCrateMenu);
+            bool locked = crateDao.isCrateLocked(iCrateId);
+            pAction->setEnabled(!locked);
             m_pCrateMenu->addAction(pAction);
             m_crateMapper.setMapping(pAction, iCrateId);
             connect(pAction, SIGNAL(triggered()), &m_crateMapper, SLOT(map()));
@@ -380,9 +441,15 @@ void WTrackTableView::contextMenuEvent(QContextMenuEvent * event)
         m_pMenu->addMenu(m_pCrateMenu);
     }
 
+    bool locked = modelHasCapabilities(TrackModel::TRACKMODELCAPS_LOCKED);
+    m_pRemoveAct->setEnabled(!locked);
     m_pMenu->addSeparator();
-    m_pMenu->addAction(m_pRemoveAct);
-
+    if (modelHasCapabilities(TrackModel::TRACKMODELCAPS_REMOVE)) {
+        m_pMenu->addAction(m_pRemoveAct);
+    }
+    if (modelHasCapabilities(TrackModel::TRACKMODELCAPS_RELOADMETADATA)) {
+        m_pMenu->addAction(m_pReloadMetadataAct);
+    }
     m_pPropertiesAct->setEnabled(oneSongSelected);
     m_pMenu->addAction(m_pPropertiesAct);
 
@@ -409,11 +476,44 @@ void WTrackTableView::onSearchCleared() {
     }
 }
 
-void WTrackTableView::onShow()
-{
-
+void WTrackTableView::onShow() {
 }
 
+void WTrackTableView::mouseMoveEvent(QMouseEvent* pEvent) {
+   Q_UNUSED(pEvent);
+   TrackModel* trackModel = getTrackModel();
+    if (!trackModel)
+        return;
+
+    // Iterate over selected rows and append each item's location url to a list
+    QList<QUrl> locationUrls;
+    QModelIndexList indices = selectionModel()->selectedRows();
+    foreach (QModelIndex index, indices) {
+        if (!index.isValid()) {
+            continue;
+        }
+        QUrl url = QUrl::fromLocalFile(trackModel->getTrackLocation(index));
+        if (!url.isValid()) {
+            qDebug() << this << "ERROR: invalid url" << url;
+            continue;
+        }
+        locationUrls.append(url);
+    }
+
+    if (locationUrls.empty()) {
+        return;
+    }
+
+    QMimeData* mimeData = new QMimeData();
+    mimeData->setUrls(locationUrls);
+
+    QDrag* drag = new QDrag(this);
+    drag->setMimeData(mimeData);
+    drag->setPixmap(QPixmap(":images/library/ic_library_drag_and_drop.png"));
+    drag->exec(Qt::CopyAction);
+}
+
+
 /** Drag enter event, happens when a dragged item hovers over the track table view*/
 void WTrackTableView::dragEnterEvent(QDragEnterEvent * event)
 {
@@ -449,8 +549,10 @@ void WTrackTableView::dragEnterEvent(QDragEnterEvent * event)
  *  Why we need this is a little vague, but without it, drag-and-drop just doesn't work.
  *  -- Albert June 8/08
  */
-void WTrackTableView::dragMoveEvent(QDragMoveEvent * event)
-{
+void WTrackTableView::dragMoveEvent(QDragMoveEvent * event) {
+    // Needed to allow auto-scrolling
+    WLibraryTableView::dragMoveEvent(event);
+
     //qDebug() << "dragMoveEvent" << event->mimeData()->formats();
     if (event->mimeData()->hasUrls())
     {
@@ -671,27 +773,19 @@ bool WTrackTableView::modelHasCapabilities(TrackModel::CapabilitiesFlags capabil
             (trackModel->getCapabilities() & capabilities) == capabilities;
 }
 
-void WTrackTableView::keyPressEvent(QKeyEvent* event)
-{
-    if (event->key() == Qt::Key_Return)
-    {
-		/*
-		 * It is not a good idea if 'key_return'
-		 * causes a track to load since we allow in-line editing
-		 * of table items in general
-		 */
+void WTrackTableView::keyPressEvent(QKeyEvent* event) {
+    if (event->key() == Qt::Key_Return) {
+        // It is not a good idea if 'key_return'
+        // causes a track to load since we allow in-line editing
+        // of table items in general
         return;
-    }
-    else if (event->key() == Qt::Key_BracketLeft)
-    {
+    } else if (event->key() == Qt::Key_BracketLeft) {
         loadSelectionToGroup("[Channel1]");
-    }
-    else if (event->key() == Qt::Key_BracketRight)
-    {
+    } else if (event->key() == Qt::Key_BracketRight) {
         loadSelectionToGroup("[Channel2]");
-    }
-    else
+    } else {
         QTableView::keyPressEvent(event);
+    }
 }
 
 void WTrackTableView::loadSelectedTrack() {
@@ -706,8 +800,9 @@ void WTrackTableView::loadSelectedTrackToGroup(QString group) {
 }
 
 void WTrackTableView::slotSendToAutoDJ() {
-    if (!modelHasCapabilities(TrackModel::TRACKMODELCAPS_ADDTOAUTODJ))
+    if (!modelHasCapabilities(TrackModel::TRACKMODELCAPS_ADDTOAUTODJ)) {
         return;
+    }
 
     PlaylistDAO& playlistDao = m_pTrackCollection->getPlaylistDAO();
     int iAutoDJPlaylistId = playlistDao.getPlaylistIdFromName(AUTODJ_TABLE);
@@ -730,6 +825,35 @@ void WTrackTableView::slotSendToAutoDJ() {
     }
 }
 
+void WTrackTableView::slotReloadTrackMetadata() {
+    if (!modelHasCapabilities(TrackModel::TRACKMODELCAPS_RELOADMETADATA)) {
+        return;
+    }
+
+    if (QMessageBox::warning(
+        NULL, tr("Reload Track Metadata"),
+        tr("Reloading track metadata on a loaded track may cause abrupt volume changes. Are you sure?"),
+        QMessageBox::Yes | QMessageBox::No,
+        QMessageBox::No) == QMessageBox::No) {
+        return;
+    }
+
+    QModelIndexList indices = selectionModel()->selectedRows();
+
+    TrackModel* trackModel = getTrackModel();
+
+    if (trackModel == NULL) {
+        return;
+    }
+
+    foreach (QModelIndex index, indices) {
+        TrackPointer pTrack = trackModel->getTrack(index);
+        if (pTrack) {
+            pTrack->parse();
+        }
+    }
+}
+
 void WTrackTableView::addSelectionToPlaylist(int iPlaylistId) {
     PlaylistDAO& playlistDao = m_pTrackCollection->getPlaylistDAO();
     TrackModel* trackModel = getTrackModel();
@@ -790,6 +914,8 @@ void WTrackTableView::doSortByColumn(int headerSection) {
         visibleColumn++;
     }
 
+    currentSelection->reset(); // remove current selection
+
     QModelIndex first;
     foreach (int trackId, trackIds) {
 
