@@ -6,18 +6,13 @@
 #include "widget/wtracktableviewheader.h"
 #include "library/trackmodel.h"
 
+#define WTTVH_MINIMUM_SECTION_SIZE 20
+
 WTrackTableViewHeader::WTrackTableViewHeader(Qt::Orientation orientation,
                                              QWidget* parent)
         : QHeaderView(orientation, parent),
           m_menu(tr("Show or hide columns."), this),
           m_signalMapper(this) {
-
-    // Show the sort indicator (technically redundant since setSortingEnabled()
-    // on a View will handle this)
-    setSortIndicatorShown(true);
-    //Allow the columns to be reordered.
-    setMovable(true);
-
     connect(&m_signalMapper, SIGNAL(mapped(int)),
             this, SLOT(showOrHideColumn(int)));
 }
@@ -60,6 +55,11 @@ void WTrackTableViewHeader::setModel(QAbstractItemModel* model) {
     // Restore saved header state to get sizes, column positioning, etc. back.
     restoreHeaderState();
 
+    // Here we can override values to prevent restoring corrupt values from database
+    setMovable(true);
+    setCascadingSectionResizes(true);
+    setMinimumSectionSize(WTTVH_MINIMUM_SECTION_SIZE);
+
     int columns = model->columnCount();
     for (int i = 0; i < columns; ++i) {
         if (trackModel->isColumnInternal(i)) {
@@ -89,6 +89,13 @@ void WTrackTableViewHeader::setModel(QAbstractItemModel* model) {
         connect(action, SIGNAL(triggered()),
                 &m_signalMapper, SLOT(map()));
         m_menu.addAction(action);
+
+        // force the section size to be a least WTTVH_MINIMUM_SECTION_SIZE
+        if (sectionSize(i) <  WTTVH_MINIMUM_SECTION_SIZE) {
+            // This might happen if  WTTVH_MINIMUM_SECTION_SIZ has changed or
+            // the header state from database was corrupt
+            resizeSection(i,WTTVH_MINIMUM_SECTION_SIZE);
+        }
     }
 
     // Safety check against someone getting stuck with all columns hidden
